## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$Id: toolbox.py 43824 2007-06-15 17:08:16Z glenfant $
"""

__author__  = ''
__docformat__ = 'restructuredtext'

# Python imports
from StringIO import StringIO
import Globals
import sys
import os

# CMF imports
from Products.CMFCore.utils import getToolByName

def check_filesystem(self, types=None, path=None):
    """Check all files. Is a file associated with an ATCT.
    Warning: Make sure uid_catalog is ok.
    """
    
    out = StringIO()
    portal_uids = []
    fs_uids = []
    utool = getToolByName(self, 'uid_catalog')
    
    if path is None:
        path = os.path.join(Globals.INSTANCE_HOME, 'var')
    
    if types is None:
        types = ['File', 'PloneExFile']
    
    out.write('Begin analyze in %s.\n' % path)
    
    # Get all uids from portal
    brains = utool(portal_type=types)
    
    for brain in brains:
        portal_uids.append(brain.UID)
    
    # Walk into filesystem
    for root, dirs, files in os.walk(path):
        if root == path:
            # Loop on files
            for item in files:
                words = item.split('_')
                
                if len(words) == 2 and len(words[0]) == 32:
                    uid = words[0]
                    fs_uids.append(uid)
    
    # Check portal uids not in fs uids
    out.write('Check portal uids not in filesystem uids.\n')
    errors_count = 0
    
    for uid in portal_uids:
        if uid not in fs_uids:
            out.write('%s failed.\n' % uid)
            errors_count += 1
    
    oks_count = len(portal_uids) - errors_count
    out.write('%d OK, %d FAILED.\n\n' % (oks_count, errors_count))
    
    
    # Check fs uids not in portal uids
    out.write('Check filesystem uids not in portal uids.\n')
    errors_count = 0
    
    for uid in fs_uids:
        if uid not in portal_uids:
            out.write('%s failed.\n' % uid)
            errors_count += 1
    
    oks_count = len(fs_uids) - errors_count
    out.write('%d OK, %d FAILED.\n\n' % (oks_count, errors_count))
    
    out.write('Analyze completed.\n')
    return out.getvalue()



# -*- coding: utf-8 -*-
## Copyright (C) 2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Configuration schema resources
$Id: datatypes.py 66391 2008-06-09 17:38:35Z glenfant $
"""

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import os
import types
from ZConfig.datatypes import stock_datatypes
from ZConfig.substitution import substitute

def existingStoragePath(value):
    """Validating/converting a storage path
    @param value: a storage path
    @return: valid (translated?) storage path
    """

    return _existingPath(value, 'fss_storage')

def existingBackupPath(value):
    """Validating/converting a backup path
    @param value: a backup path
    @return: valid (translated?) storage path
    """

    return _existingPath(value, 'fss_backup')


# ZConfig.substitution.substitute requires lowercase keys in mapping
_environ = dict([(k.lower(), v) for k, v in os.environ.items()])

# For checking duplicate paths
_paths = []

def _existingPath(value, default):

    global _environ, _paths

    # Getting the real path
    if not value:
        value = os.path.join(os.environ['INSTANCE_HOME'], 'var', default)
    else:
        value = substitute(value, _environ)

    # Check existence through ZConfig datatypes
    existing_directory = stock_datatypes['existing-directory']
    value = existing_directory(value)

    # Read/write enabled ?
    if not os.access(value, os.R_OK | os.W_OK):
        raise ValueError, "Zope process user cannot read+write in %s." % value

    # Must be unique
    if value in _paths:
        raise ValueError, "Path %s is used twice" % value
    _paths.append(value)
    return value


def default_strategy(value):
    """Validating/converting a default storage strategy
    @param value: as sent from ZConfig
    @return: valid strategy name
    """

    possible_values = ('flat', 'directory')
    value = str(value).lower()
    if value not in possible_values:
        raise ValueError("'%s' is not a valid storage strategy" % value)
    return value


def strategy(value):
    """Validating/converting a storage strategy
    @param value: as sent from ZConfig
    @return: valid strategy name
    """

    possible_values = ('flat', 'directory', 'site1', 'site2')
    value = str(value).lower()
    if value not in possible_values:
        raise ValueError("'%s' is not a valid storage strategy" % value)
    return value


class BaseConfig(object):
    """Configuration section
    """

    def __init__(self, section):
        """New (Plone) site config
        @param section: ZConfig.matcher.SectionValue obj
        """

        self._section = section
        self.name = section.getSectionName()
        self._section_attr_names = section.getSectionAttributes()
        return


    def __getattr__(self, attrname):
        """attributes are found in self.section"""

        if attrname in self._section_attr_names:
            return getattr(self._section, attrname)
        else:
            raise AttributeError, attrname


class GlobalConfig(BaseConfig):
    """Instance wide zconfig object"""

    _is_global = True

    def usesGlobalConfig(self, site_or_path):
        """Does site use the global configuration (true)
        @param site_or_path: Plone site obje or its path
        @return: backup path
        """

        return self._configForPath(site_or_path)._is_global


    def storagePathForSite(self, site_or_path):
        """Specific or global storage path
        @param site_or_path: Plone site obje or its path
        @return: storage path
        """

        return self._configForPath(site_or_path).storage_path


    def backupPathForSite(self, site_or_path):
        """Specific or global backup path
        @param site_or_path: Plone site obje or its path
        @return: backup path
        """

        return self._configForPath(site_or_path).backup_path


    def storageStrategyForSite(self, site_or_path):
        """Specific or global storage strategy
        @param site_or_path: Plone site obje or its path
        @return: storage policy
        """

        return self._configForPath(site_or_path).storage_strategy


    def _configForPath(self, site_or_path):
        """A configuration obj suitable to the site or path
        @param site_or_path: Plone site object or path to a Plone site
        This should be a decorator func but we need to run with Python 2.3
        """

        if type(site_or_path) is types.StringType:
            path = site_or_path.lower()
        else:
            path = '/'.join(site_or_path.getPhysicalPath()).lower()

        # Accelerator mapping for further calls
        if not hasattr(self, '__path_conf_map__'):
            self.__path_conf_map__ = dict([(site.name, site)
                                           for site in self.sites])

        # Specific config if found or flobal one
        return self.__path_conf_map__.get(path, self)


class SiteConfig(BaseConfig):
    """Site wide zconfig object"""

    _is_global = False



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
The storage definition
$Id: FileSystemStorage.py 66036 2008-06-02 14:19:27Z glenfant $
"""
__author__  = ''
__docformat__ = 'restructuredtext'

# Python imports
import os
from Acquisition import aq_base
from StringIO import StringIO
from types import StringType, UnicodeType

# Zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.Image import File
from OFS.SimpleItem import SimpleItem
from ZPublisher.Iterators import filestream_iterator
from zope.component import getUtility

#from Shared.DC.ZRDB.TM import TM

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.interfaces.field import IObjectField
from Products.Archetypes.Storage import StorageLayer
from Products.Archetypes.interfaces.base import IBaseUnit
from Products.Archetypes.Field import ImageField
from Products.Archetypes.Field import Image # Changes since AT1.3.4

# Products imports
from iw.fss.rdf import RDFWriter
from iw.fss.utils import copy_file
from iw.fss.interfaces import IConf

from ZPublisher.Iterators import IStreamIterator
from ZPublisher.HTTPRangeSupport import parseRange

class range_filestream_iterator(file):
    """
    a file subclass which implements an iterator that returns a
    fixed-sized sequence of bytes.
    this filestream iterator return only file portion specify by
    http range header
    A range header is send by client when he wants a portion file

    the request looks like

    GET /file.pdf HTTP/1.1
    Range: bytes=0-10
    Host: host

    And the response must have range of bit specified by the range header

    HTTP/1.1 206 Partial Content
    Accept-Ranges: bytes
    Content-Length: 11
    Content-Range: bytes 0-10/1362006
    Content-Type: application/pdf

    PDF%1.6

    The default filestream_iterator does not support this range

    """

    __implements__ = (IStreamIterator,)

    def __init__(self, name, start, end,mode='r', bufsize=-1, streamsize=1<<16):
        """
        @param :
        name  : file name in filesystem
        start : begin of the range
        end   : end of the range
        """
        self.end = end
        file.__init__(self, name, mode, bufsize)
        self.seek(start, 0)
        self.streamsize = streamsize

    def next(self):
        """
        raise a stopIteration if read bytes is upper than end value specified
        by the range validator
        """

        if self.tell() >= self.end:
            ## nothing to read
            raise StopIteration

        if self.tell() + self.streamsize > self.end:
            ## case of we must stop to the end buffer
            data = self.read(self.end - self.tell())
            if not data:
                raise StopIteration
            return data

        else:
            ## normaly do the job like filestream_iterator
            data = self.read(self.streamsize)
            if not data:
                raise StopIteration
            return data

    def __len__(self):
        """
        return len of the file
        """

        cur_pos = self.tell()
        self.seek(0, 2)
        size = self.tell()
        self.seek(cur_pos, 0)

        return size


class VirtualData:
    """
    Base abstract class for data stored on filesystem.
    Subclasses must have a docstring if they are to be published (like images)
    """
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, name, instance, path):
        self.name = name
        self.instance = instance
        self.path = path
        self.__name__ = name

    def getData(self):
        """
            Returns data (as a string) stored on filesystem.
        """

        # Use StringIO for large files
        virtual_file = StringIO()
        value = ''

        # Make sure file exists. If not returns empty string
        if not os.path.exists(self.path) or os.path.getsize(self.path) == 0:
            return ''

        # Copy all data in one pass
        try:
            copy_file(self.path, virtual_file)
            virtual_file.seek(0)
            value = virtual_file.getvalue()
        finally:
            virtual_file.close()

        return value

    def __str__(self):
        return str(self.getData())

    def __len__(self):
        return len(str(self))

    def __getattr__(self, key):
        if key == 'data':
            return self.getData()

    read = __str__

InitializeClass(VirtualData)

class VirtualBinary(VirtualData):
    """
    Base class for binary data
    """

    __content_class__ = None

    def __init__(self, name, instance, path, filename, mimetype, size):
        VirtualData.__init__(self, name, instance, path)
        self.filename = filename
        self.content_type = mimetype
        self.size = size

    def __len__(self):
        return self.size

    def absolute_url(self):
        url = '%(instance_url)s/fss_get/%(name)s' % {
            'instance_url': self.instance.absolute_url(),
            'name': self.name,
            }

        return url

    def index_html(self, REQUEST, RESPONSE=None):
        """Default view for VirtualBinary file"""
        ranges = None
        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE

        if self.__content_class__ is None:
            # Build your own headers
            RESPONSE.setHeader('Content-Type', self.content_type)
            RESPONSE.setHeader('Content-Length', self.size)

        else:
            # Call index_html method of content class with a fake
            # self.data attribute which will return empty value
            # Use this artifice to make sure content class is not loading
            # all data in memory since it is better to return a stream iterator
            # There is an exception with multiple range
            if REQUEST.environ.has_key('HTTP_RANGE'):
                ranges = parseRange(REQUEST.environ.get('HTTP_RANGE'))
             ## in case of mutiple range we don't know do with an iterator
            if ranges is not None and len(ranges) > 1:
                ## call normally OFS.image with data
                return self.__content_class__.index_html(self, REQUEST, RESPONSE)
            else:
                ## call OFS.image with no data
                setattr(self, 'data', '')
                self.__content_class__.index_html(self, REQUEST, RESPONSE)
                delattr(self, 'data')

            # This is a default header that can be bypassed by other products
            # such as attachment field.
            if RESPONSE.getHeader('content-disposition') is None: # headers are in lower case in HTTPResponse
                RESPONSE.setHeader(
                    'Content-Disposition',
                    'inline; filename="%s"' % self.filename
                    )
        if ranges and len(ranges) == 1:
            ## is an range request with one range , return an iterator with this range
            [(start,end)] = ranges
            iterator = range_filestream_iterator(self.path,start, end,  mode='rb')
            return iterator
        else:
            return filestream_iterator(self.path, mode='rb')

    def download(self, REQUEST, RESPONSE=None):
        """Download file as an attachment"""

        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE

        value = self.index_html(REQUEST, RESPONSE)

        # Change Content-Disposition header
        RESPONSE.setHeader(
            'Content-Disposition',
            'attachment; filename="%s"' % self.filename
        )
        return value

    def get_size(self):
        return len(self)

    def getContentType(self):
        return self.content_type

    def evalCmd(self, cmd_name):
        """Eval command on storage"""

        if cmd_name not in dir(self):
            raise AttributeError, 'Unknown attribute: %s' % cmd_name

        request = self.REQUEST
        response = request.RESPONSE
        kwargs = dict(request.form)
        kwargs['REQUEST'] = request
        kwargs['RESPONSE'] = response
        cmd = getattr(self, cmd_name)
        return cmd(**kwargs)

InitializeClass(VirtualBinary)


class VirtualFile(VirtualBinary, File):
    """
    For files.
    """

    __content_class__ = File
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, name, instance, path, filename, mimetype, size):
        VirtualBinary.__init__(self, name, instance, path, filename, mimetype, size)

    def __getattr__(self, key):
        if key == 'data':
            return self.getData()
        ### useless since __getattr__ is only called when no attribute is found
        ### in this class but also in parent classes. So if we are here, neither
        ### VirtualFile nor VirtualBinary and File have the requested attribute
        ### Moreover, File class has no __getattr__ method...
        # return File.__getattr__(self, key)
        raise AttributeError(key)

InitializeClass(VirtualFile)

class VirtualImage(VirtualBinary, Image):
    """
    Image objects can be GIF, PNG or JPEG and have the same methods
    as File objects. Images also have a string representation that
    renders an HTML 'IMG' tag.
    """

    __content_class__ = Image
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, name, instance, path, filename, mimetype, size, width, height):
        VirtualBinary.__init__(self, name, instance, path, filename, mimetype, size)
        self.width = width
        self.height = height

    def __getattr__(self, key):
        if key == 'data':
            return self.getData()
        return Image.__getattr__(self, key)

InitializeClass(VirtualImage)


# ####
# Storage info
# ####

class FSSInfo(SimpleItem):
    """FileSystemStorageInfo Base class. Used for string data"""

    security = ClassSecurityInfo()

    def __init__(self, uid, version=None):
        self.update(uid, version)

    security.declarePrivate('update')
    def update(self, uid, version=None):
        self.uid = uid
        self.version = version

    security.declarePrivate('getUID')
    def getUID(self):
        return self.uid

    security.declarePrivate('setUID')
    def setUID(self, uid):
        self.uid = uid

    security.declarePrivate('getVersion')
    def getVersion(self):
        return getattr(self, 'version', None)

    security.declarePrivate('setVersion')
    def setVersion(self, version):
        self.version = version

    security.declarePrivate('getProperties')
    def getProperties(self):
        """Returns info attributes in a dictionnary"""
        props = {}
        props['uid'] = self.uid
        props['version'] = self.getVersion()
        return props

    security.declarePrivate('getValue')
    def getValue(self, name, instance, path):
        return str(VirtualData(name, instance, path))

    security.declarePrivate('getRDFFieldProperties')
    def getRDFFieldProperties(self, name, instance):
        """Returns RDF field properties list.

        Each property is defined in a dictionnary {'id': ...,  'value': ...}

        @param name: name of the field
        @param instance: Content using this storage"""

        props = (
            {'id': 'dc:title', 'value': instance.title_or_id()},
            {'id': 'dc:description', 'value': instance.Description()},
            {'id': 'dc:language', 'value': instance.Language()},
            {'id': 'dc:creator', 'value': instance.Creator()},
            {'id': 'dc:date', 'value': instance.modified()},
            {'id': 'dc:format', 'value': getattr(self, 'mimetype', 'text/plain')},
            )
        return props

    security.declarePrivate('getRDF')
    def getRDF(self, name, instance):
        """Returns RDF dictionnary to inject into RDFWriter

        @param name: name of the field
        @param instance: Content using this storage"""

        rdf_args = {}

        # Get charset
        ptool = getToolByName(instance, 'portal_properties')
        rdf_args['charset'] = ptool.site_properties.default_charset

        # Get namespaces
        rdf_args['namespaces'] = (
            {'id': 'xmlns:rdf', 'value': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'},
            {'id': 'xmlns:dc', 'value': 'http://purl.org/dc/elements/1.1/'},
            {'id': 'xmlns:fss', 'value': 'http://namespace.ingeniweb.com/fss'},)

        # Get field url
        utool = getToolByName(instance, 'portal_url')
        portal_path = utool.getPortalObject().getPhysicalPath()
        portal_path_len = len(portal_path)
        rel_path = '/'.join(instance.getPhysicalPath()[portal_path_len:])
        fss_path = '%s/fss_get/%s' % (rel_path, name)
        rdf_args['field_url'] = fss_path

        # Get field properties
        rdf_args['field_props'] = self.getRDFFieldProperties(name, instance)
        return rdf_args

    security.declarePrivate('getRDFValue')
    def getRDFValue(self, name, instance, rdf_script=''):
        """Returns RDF value of file.

        Use this method if FSSInfo is mentionned

        @param name: Name of the field
        @param instance: Content using this storage
        @param rdf_script: Script used to generate rdf args
        """

        default_rdf = self.getRDF(name, instance)
        rdf_args = None
        if rdf_script:
            func = getattr(instance, rdf_script, None)
            if func is not None:
                rdf_args = func(name=name, instance=instance, properties=self.getProperties(), default_rdf=default_rdf)

        if rdf_args is None:
            rdf_args = default_rdf
        writer = RDFWriter(**rdf_args)
        return writer.getRDF()

InitializeClass(FSSInfo)

class FSSFileInfo(FSSInfo):
    """FileSystemStorageInfo File class. Used for file data"""

    security = ClassSecurityInfo()

    def __init__(self, uid, title, size, mimetype, version=None):
        self.update(uid, title, size, mimetype, version)

    security.declarePrivate('update')
    def update(self, uid, title, size, mimetype, version=None):
        FSSInfo.update(self, uid, version)
        self.title = title
        self.size = size
        self.mimetype = mimetype

    security.declarePrivate('getTitle')
    def getTitle(self):
        return self.title

    security.declarePrivate('setTitle')
    def setTitle(self, title):
        self.title = title

    security.declarePrivate('getSize')
    def getSize(self):
        return self.size

    security.declarePrivate('setSize')
    def setSize(self, size):
        self.size = size

    security.declarePrivate('getMimetype')
    def getMimetype(self):
        return self.mimetype

    security.declarePrivate('setMimetype')
    def setMimetype(self, mimetype):
        self.mimetype = mimetype

    security.declarePrivate('getValue')
    def getValue(self, name, instance, path):
        return VirtualFile(name, instance, path, self.title, self.mimetype, self.size)

    security.declarePrivate('getProperties')
    def getProperties(self):
        """Returns info attributes in a dictionnary"""

        props = FSSInfo.getProperties(self)
        props['mimetype'] = self.mimetype
        props['title'] = self.title
        props['size'] = self.size
        return props

    security.declarePrivate('getRDFFieldProperties')
    def getRDFFieldProperties(self, name, instance):
        """Returns RDF field properties list.

        Each property is defined in a dictionnary {'id': ...,  'value': ...}

        @param name: name of the field
        @param instance: Content using this storage"""

        props = FSSInfo.getRDFFieldProperties(self, name, instance) + (
            {'id': 'fss:filename', 'value': self.title},
            {'id': 'fss:size', 'value': self.size},
            )
        return props

InitializeClass(FSSFileInfo)

class FSSImageInfo(FSSFileInfo):
    """FileSystemStorageInfo Image class. Used for image data"""

    security = ClassSecurityInfo()

    def __init__(self, uid, title, size, mimetype, width, height, version=None):
        self.update(uid, title, size, mimetype, width, height, version)

    security.declarePrivate('update')
    def update(self, uid, title, size, mimetype, width, height, version=None):
        FSSFileInfo.update(self, uid, title, size, mimetype, version)
        self.width = width
        self.height = height

    security.declarePrivate('getWidth')
    def getWidth(self):
        return self.width

    security.declarePrivate('setWidth')
    def setWidth(self, width):
        self.width = width

    security.declarePrivate('getHeight')
    def getHeight(self):
        return self.height

    security.declarePrivate('setHeight')
    def setHeight(self, height):
        self.height = height

    security.declarePrivate('getValue')
    def getValue(self, name, instance, path):
        return VirtualImage(name, instance, path, self.title, self.mimetype, self.size, self.width, self.height)

    security.declarePrivate('getProperties')
    def getProperties(self):
        """Returns info attributes in a dictionnary"""

        props = FSSFileInfo.getProperties(self)
        props['width'] = self.width
        props['height'] = self.height
        return props

    security.declarePrivate('getRDFFieldProperties')
    def getRDFFieldProperties(self, name, instance):
        """Returns RDF field properties list.

        Each property is defined in a dictionnary {'id': ...,  'value': ...}

        @param name: name of the field
        @param instance: Content using this storage"""

        props = FSSFileInfo.getRDFFieldProperties(self, name, instance) + (
            {'id': 'fss:width', 'value': self.width},
            {'id': 'fss:height', 'value': self.height},
            )
        return props

InitializeClass(FSSImageInfo)

# Keep it for compatibility
class FileSystemStorageInfo(FSSFileInfo):
    pass

InitializeClass(FileSystemStorageInfo)


# ####
# Storage
# ####

class FileSystemStorage(StorageLayer):

    __implements__ = StorageLayer.__implements__

    security = ClassSecurityInfo()


    security.declarePrivate('getFSSInfoVarname')
    def getFSSInfoVarname(self, name):
        """ """

        return '%s_filesystemstorage_info' % name


    security.declarePrivate('getFSSInfo')
    def getFSSInfo(self, name, instance, **kwargs):
        """Get fss info"""

        info_varname = self.getFSSInfoVarname(name)
        return getattr(aq_base(instance), info_varname, None)


    security.declarePrivate('delFSSInfo')
    def delFSSInfo(self, name, instance, **kwargs):
        """Delete fss info attribute"""

        info_varname = self.getFSSInfoVarname(name)
        delattr(aq_base(instance), info_varname)


    security.declarePrivate('setFSSInfo')
    def setFSSInfo(self, name, instance, value, **kwargs):
        """Set new value in fss info"""

        info_varname = self.getFSSInfoVarname(name)
        field = self.getField(name, instance, **kwargs)

        # Check types
        uid = instance.UID()
        version = kwargs.get('version')

        if isinstance(value, File) or isinstance(value, Image):
            size = value.get_size()
            mimetype = kwargs.get('mimetype', getattr(value, 'content_type', 'application/octet-stream'))
            title = kwargs.get('filename', getattr(value, 'filename', getattr(value, 'title', name)))

            if isinstance(value, Image):
                width = value.width
                height = value.height
                if type(title) not in (StringType, UnicodeType):
                    # Make sure title is a string (Fix for image thumbs)
                    title = name

                # Like Image field
                info = FSSImageInfo(uid, title, size, mimetype, width, height,
                                    version)
            else:
                # Like File field
                info = FSSFileInfo(uid, title, size, mimetype, version)
        else:
            # Other
            info = FSSInfo(uid, version)

        # Make sure, we have deleted old info
        if hasattr(aq_base(instance), info_varname):
            delattr(instance, info_varname)

        # Store in attributes
        setattr(instance, info_varname, info)

        return info


    security.declarePrivate('getField')
    def getField(self, name, instance, **kwargs):
        """Get field"""

        return kwargs.get('field', instance.getField(name))

    def getInheritedNames(self, instance, field):
        """Returns all names derivating from a field

        Include field name it self. Then for example Image field size names
        """

        name = field.getName()

        # List of names created from one Field
        # Include field name
        # ImageField has size names (image_mini, image_thumb)
        names = [name]

        # This is a workaround for scales in ImageField
        if isinstance(field, ImageField):
            names.extend(['%s_%s' % (name, x) for x in field.getAvailableSizes(instance).keys()])

        return names

    def getConf(self):
        conf = getUtility(IConf, "globalconf")
        return conf()

    def getStorageStrategy(self, name, instance):
        """Get strategy that defined how field values are stored"""
        return self.getConf().getStorageStrategy()

    def getStorageStrategyProperties(self, name, instance, info):
        """Returns a dictionnary containing all properties used by
        strategies to store field values"""

        kwargs = {}
        kwargs['name'] = name
        utool = getToolByName(instance, 'portal_url')
        kwargs['path'] = '/'.join(utool.getRelativeContentPath(instance))
        kwargs.update(info.getProperties())
        return kwargs


    security.declarePrivate('get')
    def get(self, name, instance, **kwargs):
        """Get value of a field"""

        info = self.getFSSInfo(name, instance)

        if info is None:
            return ''

        strategy = self.getStorageStrategy(name, instance)
        props = self.getStorageStrategyProperties(name, instance, info)

        # Restore backup if exists
        strategy.restoreValueFile(**props)
        path = strategy.getValueFilePath(**props)
        return info.getValue(name, instance, path)

    security.declarePrivate('set')
    def set(self, name, instance, value, **kwargs):
        """Set value of a field"""

        # Ignore initialize process
        initializing = kwargs.get('_initializing_', False)
        if initializing:
            return

        # Remove acquisition wrappers
        value = aq_base(value)

        # Create File System Storage Info
        info = self.setFSSInfo(name, instance, value, **kwargs)

        # Wrap value
        if IObjectField.isImplementedBy(value):
            value = value.getRaw(self.instance)
        if IBaseUnit.isImplementedBy(value):
            value = value.getRaw()
        elif isinstance(value, File):
            value = value.data
        else:
            value = str(value)

        # Copy file on filesystem
        strategy = self.getStorageStrategy(name, instance)
        props = self.getStorageStrategyProperties(name, instance, info)
        strategy.setValueFile(value, **props)

        # Create RDF file
        conf = self.getConf()
        is_rdf_enabled = conf.isRDFEnabled()
        rdf_script = conf.getRDFScript()

        if is_rdf_enabled:
            # Replace rdf file
            rdf_value = info.getRDFValue(name, instance, rdf_script)
            strategy.setRDFFile(rdf_value, **props)

    security.declarePrivate('restoreValueFile')
    def restoreValueFile(self, strategy, name, instance):
        """ restore all versions """

        to_restore = []
        rtool = getToolByName(instance, 'portal_repository')
        history = rtool.getHistory(instance, oldestFirst=True)

        if (len(history)):
            for version in history:
                to_restore.append(version.object.__of__(instance.aq_parent))

        to_restore.append(instance)

        for obj in to_restore:
            info = self.getFSSInfo(name, obj)
            props = self.getStorageStrategyProperties(name, obj, info)
            strategy.restoreValueFile(**props)


    security.declarePrivate('unset')
    def unset(self, name, instance, **kwargs):
        """Delete all versions"""

        to_unset = []
        rtool = getToolByName(instance, 'portal_repository')
        history = rtool.getHistory(instance, oldestFirst=True)

        if (len(history)):
            for version in history:
                to_unset.append(version.object.__of__(instance.aq_parent))

        to_unset.append(instance)

        for obj in to_unset:
            self.unsetVersion(name, obj, **kwargs)


    def unsetVersion(self, name, instance, **kwargs):
        """Delete field value"""

        info = self.getFSSInfo(name, instance)

        if info is None:
            return

        # This method can be called when an object is cut/paste
        strategy = self.getStorageStrategy(name, instance)
        props = self.getStorageStrategyProperties(name, instance, info)
        is_moved = kwargs.get('is_moved', False)
        strategy.unsetValueFile(is_moved=is_moved, **props)
        strategy.unsetRDFFile(is_moved=is_moved, **props)

        # Delete field so delete fss attribute
        if not is_moved:
            # Store fss info in a volatile because it could be just a move
            # Check it in cleanupInstance method
            info = self.getFSSInfo(name, instance, **kwargs)
            fss_props = getattr(instance, '_v_fss_props', {})
            fss_props[name] = info
            setattr(instance, '_v_fss_props', fss_props)

            # Delete FSS info
            self.delFSSInfo(name, instance, **kwargs)


    security.declarePrivate('initializeField')
    def initializeField(self, instance, field, **kwargs):
        """Initialize field of object"""

        # Remove _v_fss_move attribute
        if hasattr(instance, '_v_fss_move'):
            delattr(instance, '_v_fss_move')

        name = field.getName()
        info = self.getFSSInfo(name, instance)

        if info is None:
            return

        names = self.getInheritedNames(instance, field)
        uid = instance.UID()
        src_uid = info.getUID()
        conf = self.getConf()
        is_rdf_enabled = conf.isRDFEnabled()
        rdf_script = conf.getRDFScript()
        strategy = self.getStorageStrategy(name, instance)

        if uid == src_uid:
            # Cut/Paste
            for name in names:
                self.restoreValueFile(strategy, name, instance)

                if is_rdf_enabled:
                    # Replace rdf file
                    rdf_value = info.getRDFValue(name, instance, rdf_script)
                    props = self.getStorageStrategyProperties(name, instance, info)
                    strategy.setRDFFile(rdf_value, **props)
        else:
            # Copy/Paste
            # Get source object
            atool= getToolByName(instance, 'archetype_tool')
            utool = getToolByName(instance, 'portal_url')
            src_obj = atool.getObject(src_uid)

            if src_obj is not None:
                src_path = '/'.join(utool.getRelativeContentPath(src_obj))

                for name in names:
                    info = self.getFSSInfo(name, instance)
                    info.setUID(uid)
                    props = self.getStorageStrategyProperties(name, instance, info)
                    strategy.copyValueFile(src_uid=src_uid, src_path=src_path, **props)

                    if is_rdf_enabled:
                        # Replace rdf file
                        rdf_value = info.getRDFValue(name, instance, rdf_script)
                        strategy.setRDFFile(rdf_value, **props)
            else:
                # Source object does not exist. Maybe object UID has been
                # changed using _setUID method
                for name in names:
                    info = self.getFSSInfo(name, instance)
                    info.setUID(uid)
                    props = self.getStorageStrategyProperties(name, instance, info)
                    strategy.moveValueFile(src_uid=src_uid, **props)

                    if is_rdf_enabled:
                        # Replace rdf file
                        rdf_value = info.getRDFValue(name, instance, rdf_script)
                        props = self.getStorageStrategyProperties(name, instance, info)
                        strategy.setRDFFile(rdf_value, **props)

    security.declarePrivate('cleanupField')
    def cleanupField(self, instance, field, **kwargs):
        """Delete object field"""

        # Is this object deleted or just cut/paste
        is_moved = getattr(instance, '_v_cp_refs', False)

        # Create special _v_fss_move attribute to avoid event bug in AT 1.4
        if is_moved:
            setattr(instance, '_v_fss_move', True)

        is_moved = getattr(instance, '_v_fss_move', False)

        # Remove field values
        name = field.getName()
        names = self.getInheritedNames(instance, field)

        for name in names:
            self.unset(name, instance, is_moved=is_moved, **kwargs)


    security.declarePrivate('initializeInstance')
    def initializeInstance(self, instance, item=None, container=None):
        """Initialize new object"""

        # Not implemented
        pass


    security.declarePrivate('cleanupInstance')
    def cleanupInstance(self, instance, item=None, container=None):
        """Delete object"""

        if item is None:
            return

        # chekc if item is moved or really deleted
        is_moved = getattr(item, '_v_cp_refs', False)

        # If item is moved, apply, fss_move variable to instance
        if not is_moved:
            return

        fss_props = getattr(instance, '_v_fss_props', None)

        if fss_props is None:
            return

        for name, info in fss_props.items():
            fss_info_name = self.getFSSInfoVarname(name)
            setattr(instance, fss_info_name, info)


InitializeClass(FileSystemStorage)



## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: zcml.py 65108 2008-05-19 20:23:31Z cbosse $
"""
ZCML fss namespace handling, see meta.zcml
"""
__author__  = 'glenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import logging
from zope.interface import Interface
from zope.configuration.fields import GlobalObject, Tokens, PythonIdentifier

import config
from FileSystemStorage import FileSystemStorage

class ITypeWithFSSDirective(Interface):
    """Schema for fss:typeWithFSS directive"""

    class_ = GlobalObject(
        title=u'Class',
        description=u'Dotted name of class of AT based content type using FSS',
        required=True)

    fields = Tokens(
        title=u'Fields',
        description=u'Field name or space(s) separated field names',
        value_type=PythonIdentifier(),
        required=True)


def typeWithFSS(_context, class_, fields):
    """Register our monkey patch"""

    _context.action(
        discriminator=(class_.__module__,class_.__name__),
        callable=patchATType,
        args=(class_, fields)
        )


logger = logging.getLogger(config.PROJECTNAME)
LOG = logger.info

def patchATType(class_, fields):
    """Processing the type patch"""
    global patchedTypesRegistry

    for fieldname in fields:
        field = class_.schema[fieldname]
        former_storage = field.storage
        field.storage = FileSystemStorage()
        field.registerLayer('storage', field.storage)
        if patchedTypesRegistry.has_key(class_):
            patchedTypesRegistry[class_][fieldname] = former_storage
        else:
            patchedTypesRegistry[class_] = {fieldname: former_storage}
        LOG("Field '%s' of %s is stored in file system.", fieldname, class_.meta_type)
    return

# We register here the types that have been patched for migration purpose
patchedTypesRegistry = {
    # {content class : {field name: storage, ...}, ...}
    }



# -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
The FileSystemStorage package
$Id: __init__.py 66391 2008-06-09 17:38:35Z glenfant $
"""

__author__  = ''
__docformat__ = 'restructuredtext'

import os

from Products.CMFCore.utils import ContentInit
from Products.CMFCore import permissions as CCP

from Products.Archetypes.public import process_types, listTypes

from Products.CMFEditions.Modifiers import ConditionalTalesModifier

from iw.fss.config import (
    PROJECTNAME, ZOPETESTCASE, INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE)

from iw.fss.modifier import manage_addModifier
from iw.fss.modifier import modifierAddForm
from iw.fss.modifier import MODIFIER_ID

def initialize(context):


    if ZOPETESTCASE or os.environ.get(INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE):
        # Import example types
        from iw.fss import examples
        dummy = examples # No pyflakes warning
        content_types, constructors, ftis = process_types(listTypes(PROJECTNAME),
                                                          PROJECTNAME)
        ContentInit('%s Content' % PROJECTNAME,
                    content_types = content_types,
                    permission = CCP.AddPortalContent,
                    extra_constructors = constructors,
                    fti = ftis,
                    ).initialize(context)


    # Register modifier
    context.registerClass(
        ConditionalTalesModifier,
        MODIFIER_ID,
        permission=CCP.ManagePortal,
        constructors = (modifierAddForm, manage_addModifier),
        icon='modifier.gif',
        )

    # Setup module aliases to bind all Zope2 products
    import modulealiases
    dummy = modulealiases # No pyflakes warning

    # Provides 'plone' domain translations
    if not ZOPETESTCASE:
        context._ProductContext__app.Control_Panel.TranslationService._load_i18n_dir(os.path.dirname(__file__)+'/i18n')

    return



# -*- coding: utf-8 -*-
## Copyright (C) 2006 - 2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Global FileSystemStorage configuration data
$Id: config.py 66034 2008-06-02 14:17:47Z glenfant $
"""

__author__  = ''
__docformat__ = 'restructuredtext'

from customconfig import ZOPETESTCASE, INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE

PROJECTNAME = 'iw.fss'
GLOBALS = globals()
I18N_DOMAIN = PROJECTNAME.lower()
PROPERTYSHEET = 'filesystemstorage_properties'

ZCONFIG, dummy_handler, CONFIG_FILE = None, None, None

def loadConfig():
    """Loads configuration from a ZConfig file"""

    global ZCONFIG, dummy_handler, CONFIG_FILE

    import os
    from Globals import INSTANCE_HOME
    from ZConfig.loader import ConfigLoader
    from iw.fss.configuration.schema import fssSchema

    # Configuration directories
    INSTANCE_ETC = os.path.join(INSTANCE_HOME, 'etc')
    _this_directory = os.path.abspath(os.path.dirname(__file__))
    FSS_ETC = os.path.join(_this_directory, 'etc')

    def filePathOrNone(file_path):
        return os.path.isfile(file_path) and file_path or None

    # (Potential) configuration files
    CONFIG_FILENAME = 'plone-filesystemstorage.conf'
    INSTANCE_CONFIG = filePathOrNone(os.path.join(INSTANCE_ETC, CONFIG_FILENAME))
    FSS_CONFIG = filePathOrNone(os.path.join(FSS_ETC, CONFIG_FILENAME))
    FSS_CONFIG_IN = filePathOrNone(os.path.join(FSS_ETC, CONFIG_FILENAME + '.in'))

    # We configure on the first available config file
    CONFIG_FILE = [fp for fp in (INSTANCE_CONFIG, FSS_CONFIG, FSS_CONFIG_IN)
                   if fp is not None][0]

    # We ignore personal configuration on unit tests
    if ZOPETESTCASE:
        ZCONFIG, dummy_handler = ConfigLoader(fssSchema).loadURL(FSS_CONFIG_IN)
    else:
        ZCONFIG, dummy_handler = ConfigLoader(fssSchema).loadURL(CONFIG_FILE)


    # Dirty but we need to reinit datatypes control globals since this
    # initialisation seems to be called more than once with Zope 2.8
    # (why ???)
    #from iw.fss.configuration import datatypes
    #datatypes._paths = []
    #return

loadConfig()



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""Module aliases for unpickling

The dotted class path of most persistent classes were changed after version 2.6
This module creates aliases using sys.modules for unpickling old objects.
"""
__author__  = ''
__docformat__ = 'restructuredtext'

import sys

from iw import fss

sys.modules['Products.FileSystemStorage'] = fss




## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: interfaces.py 66391 2008-06-09 17:38:35Z glenfant $
"""
Interfaces exposed here
"""
__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from zope.interface import Interface
from zope.annotation.interfaces import IAttributeAnnotatable

class IFSSInfo(IAttributeAnnotatable):
    """Marker for FSSInfo"""
    pass

class IFSSTool(Interface):
    """Marker for FSS tool"""
    pass

class IConf(Interface):
    """Main configuration for fss"""
    def isRDFEnabled():
        """Returns true if RDF is automaticaly generated when file added"""
    def enableRDF(enabled):
        """Enable rdf or not"""
    def getRDFScript():
        """Returns rdf script used to generate RDF on files"""
    def setRDFScript(rdf_script):
        """Set rdf script used to generate RDF on files"""
    def getStorageStrategy():
        """Returns the storage strategy"""
    def getUIDToPathDictionnary():
        """Returns a dictionnary"""
    def getPathToUIDDictionnary():
        """Returns a dictionnary"""
    def getFSSBrains(items):
        """Returns a dictionnary."""
    def getStorageBrains():
        """Returns a list of brains in storage path"""
    def getStorageBrainsByUID(uid):
        """ Returns a list containing all brains related to fields stored
        on filesystem of object having the specified uid"""
    def getBackupBrains():
        """Returns a list of brains in backup path"""
    def updateFSS():
        """Update FileSystem storage"""
    def removeBackups(max_days):
        """Remove backups older than specified days"""
    def updateRDF():
        """Add RDF files to fss files"""
    def getFSSItem(instance, name):
        """Get value of fss item.
        This method is called from fss_get script."""
    def configletTabs(template_id):
        """Data for drawing tabs in FSS config panel"""
    def getFSStats():
        """Returns stats on FileSystem storage"""
    def patchedTypesInfo():
        """A TALES friendly summary of content types with storage changed to FSS"""
    def siteConfigInfo():
        """A TALES friendly configuration info mapping for this Plone site"""
    def globalConfigInfo():
        """A TALES friendly configuration info mapping for global configuration"""
    def formattedReadme():
        """README.txt (reStructuredText) transformed to HTML"""


class IStrategy(Interface):
    """Defines the way to store files"""

    def walkOnStorageDirectory(**kwargs):
        """Walk on storage directory"""

    def walkOnBackupDirectory(**kwargs):
        """Walk on backup directory"""

    def walkOnValueDirectoryPath(**kwargs):
        """Get path of directory where the field value is stored"""

    def getValueDirectoryPath(**kwargs):
        """Get path of directory where the field value is stored"""

    def getValueFilename(**kwargs):
        """Get filename of the field value on filesystem"""

    def getValueFilePath(**kwargs):
        """Get path of file where the field value is stored"""

    def getRDFDirectoryPath(**kwargs):
        """Get path of directory where the rdf value is stored"""

    def getRDFFilename(**kwargs):
        """Get filename of the rdf value on filesystem"""

    def getRDFFilePath(**kwargs):
        """Get path of file where the rdf value is stored"""

    def getBackupDirectoryPath(**kwargs):
        """Get path of directory where the file value is backup"""

    def getBackupFilename(**kwargs):
        """Get filename of the file backup value on filesystem"""

    def getBackupFilePath(**kwargs):
        """Get path of file where the file value is backup"""

    def setValueFile(value, **kwargs):
        """Copy file value on filesystem"""

    def unsetValueFile(**kwargs):
        """Remove file value if exists"""

    def moveValueFile(**kwargs):
        """File properties has changed, move it its new locations"""

    def restoreValueFile(**kwargs):
        """Restore the backup value if exists"""

    def copyValueFile(**kwargs):
        """Duplicate file value on filesystem"""



# -*- coding: utf-8 -*-
## ProductName
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Export FileSystemStorage tree in a site like tree
"""

__version__ = "$Revision: 1.31 $"
__docformat__ = 'restructuredtext'

# Python imports
import re
import os
import sys
import getopt
from StringIO import StringIO


MAN = """python build_fs_tree.py SOURCE_PATH DEST_PATH LIB_PATH

SOURCE_PATH
    Path where fss and rdf files are stored.

DEST_PATH
    Path where filesystem tree are built.

LIB_PATH
    Path where FileSystemStorage is installed.
"""

def usage():
    print MAN

SEARCH_RDF = r'^(?P<uid>.{32})_(?P<field>[^.]*).rdf$'
SEARCH_RDF_RE = re.compile(SEARCH_RDF)

def build_fs_tree(src_path, dst_path, lib_path):
    """Build FS tree"""

# Rdf imports
    sys.path.append(lib_path)
    from rdf import RDFReader
    from utils import copy_file

    print "Build filesystem data in %s from %s" % (src_path, dst_path)
    sys_encoding = sys.getfilesystemencoding()

    # Store rdf files
    # List of dictionnary {'field': ..., 'uid': ...}
    rdf_files = []

    # Walk into filesystem
    for root, dirs, files in os.walk(src_path):
        if root == src_path:
            # Loop on files
            for item in files:
                match = SEARCH_RDF_RE.match(item)
                if match is None:
                    continue

                # Get field name and content uid
                uid = match.group('uid')
                field = match.group('field')
                rdf_files.append({'uid': uid, 'field': field})

    # Processing collected rdf files
    print "Processing %s rdf files" % str(len(rdf_files))
    file_paths = []
    for rdf_file in rdf_files:
        uid = rdf_file['uid']
        field = rdf_file['field']

        # Get RDF file
        rdf_filename = '%s_%s.rdf' % (uid, field)
        rdf_path = os.path.join(src_path, rdf_filename)
        rdf_file = StringIO()
        rdf_text = ''
        try:
            copy_file(rdf_path, rdf_file)
            rdf_file.seek(0)
            rdf_text = rdf_file.getvalue()
        finally:
            rdf_file.close()

        # Read RDF properties
        try:
            rdf_reader = RDFReader(rdf_text)
        except:
            try:
                # XXX known bug to fix
                rdf_text = rdf_text.replace('&', '&amp;')
                rdf_reader = RDFReader(rdf_text)
            except:
                print rdf_path
                print rdf_text
                raise
        field_url = rdf_reader.getFieldUrl()
        field_url = field_url.encode(sys_encoding, 'replace')

        # Create tree directories
        content_path_array = field_url.split('/')[:-2]
        content_path = dst_path
        for content_dir in content_path_array:
            content_path = os.path.join(content_path, content_dir)
            if os.path.exists(content_path):
                continue
            print "Create path: %s" % content_path
            os.mkdir(content_path)

        # Get source file
        src_filename = '%s_%s' % (uid, field)
        src_file_path = os.path.join(src_path, src_filename)

        if not os.path.exists(src_file_path):
            print "Source file doesn't exist, we continue: %s" % src_file_path
            continue

        # Get destination file
        dst_filename = field
        dst_filenames = rdf_reader.getFieldProperty('fss:filename')
        if dst_filenames:
            dst_filename = dst_filenames[0]
            if not dst_filename:
                dst_filename = field
            else:
                dst_filename = dst_filename.encode(sys_encoding, 'replace')
        dst_file_path = os.path.join(content_path, dst_filename)

        # In some cases, you can have a content having 2 fss fields with
        # 2 files with the same name
        orig_dst_filename = dst_filename
        dst_file_path_ok = False
        index = 0
        while not dst_file_path_ok:
            if dst_file_path not in file_paths:
                dst_file_path_ok = True
                file_paths.append(dst_file_path)
            else:
                index += 1
                dst_filename = '%s-%s' % (str(index), orig_dst_filename)
                print dst_filename
                dst_file_path = os.path.join(content_path, dst_filename)

        print "Create file: %s" % dst_file_path
        copy_file(src_file_path, dst_file_path)
        print "Create RDF file: %s.rdf" % dst_file_path
        copy_file(rdf_path, dst_file_path + '.rdf')

    print "Filesystem data built complete"

def main():
    """Build FS tree"""

    # Parse command line options
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["help"])
    except getopt.error, msg:
        print msg
        print "for help use --help"
        sys.exit(2)

    # Process options
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)

    # Check args
    if len(args) != 3:
        print "for help use --help"
        usage()
        sys.exit(2)

    # Check paths
    src_path = args[0]
    dst_path = args[1]
    lib_path = args[2]

    if not os.path.exists(src_path):
        print "source path is not valid: %s" % src_path
        sys.exit(2)

    if not os.path.exists(src_path):
        print "destination path is not valid: %s" % dst_path
        sys.exit(2)

    if not os.path.exists(lib_path):
        print "lib path is not valid: %s" % lib_path
        sys.exit(2)

    # Process files
    build_fs_tree(src_path, dst_path, lib_path)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Demo content type with a file and an image.
$Id: FSSItem.py 65108 2008-05-19 20:23:31Z cbosse $
"""

# Zope imports
from AccessControl import ClassSecurityInfo

# Archetypes imports
from Products.Archetypes.public import Schema
from Products.Archetypes.public import BaseSchema
from Products.Archetypes.public import FileField
from Products.Archetypes.public import ImageField
from Products.Archetypes.public import TextField
from Products.Archetypes.public import FileWidget
from Products.Archetypes.public import ImageWidget
from Products.Archetypes.public import TextAreaWidget
from Products.Archetypes.public import PrimaryFieldMarshaller
from Products.Archetypes.public import BaseContent
from Products.Archetypes.public import registerType

from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.base import ATContentTypeSchema

# Products imports
from iw.fss.config import PROJECTNAME
from iw.fss.FileSystemStorage import FileSystemStorage

BaseItemShema = Schema((
    FileField('file',
              required=False,
              primary=True,
              storage=FileSystemStorage(),
              widget=FileWidget(
                  description="Select the file to be added by clicking the 'Browse' button.",
                  description_msgid="help_file",
                  label="File",
                  label_msgid="label_file",
                  i18n_domain="plone",
                  show_content_type=False,)),
    ImageField('image',
               required=False,
               sizes={
                   'mini':(40,40),
                   'thumb':(80,80),},
               storage=FileSystemStorage(),
               widget=ImageWidget()),
    TextField('text',
              required=False,
              storage=FileSystemStorage(),
              widget=TextAreaWidget()),
    ), marshall=PrimaryFieldMarshaller())

FSSItemSchema = BaseSchema.copy() + BaseItemShema.copy()
ATFSSItemSchema = ATContentTypeSchema.copy() + BaseItemShema.copy()

class FSSItem(BaseContent):
    """A simple item using FileSystemStorage"""
    archetypes_name = portal_type = meta_type = 'FSSItem'
    schema = FSSItemSchema
    _at_rename_after_creation = True


registerType(FSSItem, PROJECTNAME)

class ATFSSItem(ATCTContent):
    """A simple item using FileSystemStorage base on ATContentypes"""
    archetypes_name = portal_type = meta_type = 'ATFSSItem'
    schema = ATFSSItemSchema
    _at_rename_after_creation = True


registerType(ATFSSItem, PROJECTNAME)



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Testing FSS features
$Id: testFSS.py 66025 2008-06-02 13:53:50Z glenfant $
"""

from common import *
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse
from StringIO import StringIO
from zope.component import getUtility
from iw.fss.interfaces import IConf

class TestFSS(FSSTestCase.FSSTestCase):
    def afterSetUp(self):
        self.loginAsPortalOwner()
        content_id = 'test_folder'
        self.portal.invokeFactory(FOLDER_TYPE, id=content_id)
        self.test_folder = getattr(self.portal, content_id)
        self.logout()
        self.portal_repository = self.portal.portal_repository
        conf_class = getUtility(IConf, "globalconf")
        self.conf = conf_class()


# #############################################################################
# ADD
# #############################################################################

    def testAddFileFromString(self):
        self.loginAsPortalOwner()
        content_id = 'test_file'
        self.file_content = self.addFileByString(self.test_folder, content_id)

        # Get file field
        file_field = self.file_content.getField('file')

        # Get file value
        file_value = file_field.get(self.file_content)

        # Test value
        self.assertEquals(str(file_value.data), 'mytestfile')

        # Test filename
        self.assertEquals(file_value.filename, '')

        # Test size
        self.assertEquals(file_value.get_size(), 10)

        # Test content type
        self.assertEquals(file_field.getContentType(self.file_content), 'text/plain')

        # Test using BaseUnit
        bu = file_field.getBaseUnit(self.file_content)
        bu_value = bu.getRaw()
        self.assertEquals(len(bu_value), 10)
        self.assertEquals(bu_value, """mytestfile""")

        # Test using manage_FTPget
        ftp_value = self.file_content.manage_FTPget()
        self.failIf(ftp_value is None)
        self.assertEquals(len(ftp_value), 10)
        self.assertEquals(ftp_value, """mytestfile""")
        self.logout()

    def _testDefaultContentFromUploadedFile(self):
        # Get file field
        file_field = self.file_content.getField('file')

        # Get file value
        file_value = file_field.get(self.file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(self.file_content), 'application/msword')

        # Test using BaseUnit
        bu = file_field.getBaseUnit(self.file_content)
        bu_value = bu.getRaw()
        self.assertEquals(len(bu_value), 10240)

        # Test using manage_FTPget
        ftp_value = self.file_content.manage_FTPget()
        self.failIf(ftp_value is None)
        self.assertEquals(len(ftp_value), 10240)

    def testAddFileFromUploadedFile(self):
        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)

        self._testDefaultContentFromUploadedFile()

        self.logout()

# #############################################################################
# EDIT
# #############################################################################
    def testEditFile(self):
        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)

        self._testDefaultContentFromUploadedFile()

        # Update content
        data_path = self.getDataPath()
        self.updateContent(self.file_content, 'file', os.path.join(data_path, 'excel.xls'))

        # Get file field
        file_field = self.file_content.getField('file')

        # Get file value
        file_value = file_field.get(self.file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'excel.xls')

        # Test size
        self.assertEquals(file_value.get_size(), 13824)

        # Test content type
        self.assertEquals(file_field.getContentType(self.file_content), 'application/vnd.ms-excel')

        self.logout()

# #############################################################################
# RENAME
# #############################################################################
    def testRenameContent(self):
        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)
        old_uid = self.file_content.UID()

        # Rename file
        new_content_id = 'new_test_file'
        self.test_folder.manage_renameObjects((content_id,), (new_content_id,))

        # Test file
        self.assertEquals(self.file_content.getId(), new_content_id)
        self.assertEquals(self.file_content.UID(), old_uid)
        self._testDefaultContentFromUploadedFile()
        self.logout()

# #############################################################################
# COPY
# #############################################################################
    def testCopyContent(self):
        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # Copy file in another folder
        cb = self.test_folder.manage_copyObjects(ids=(content_id,))

        new_folder_id = 'new_test_folder'
        self.portal.invokeFactory(FOLDER_TYPE, id=new_folder_id)
        new_folder = getattr(self.portal, new_folder_id)

        # Paste
        new_folder.manage_pasteObjects(cb_copy_data=cb)

        # Test source file
        self._testDefaultContentFromUploadedFile()

        # Test destination file
        new_file_content = getattr(new_folder, content_id)

        # Get file field
        file_field = new_file_content.getField('file')

        # Get file value
        file_value = file_field.get(new_file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(new_file_content), 'application/msword')

        # more tests here to test that the external content
        # is also copied
        old_storage = getattr(self.test_folder, content_id).getField('file').getStorage()
        new_storage = file_field.getStorage()
        old_instance = getattr(self.test_folder, content_id)

        old_fss = old_storage.getFSSInfo('file', old_instance)
        new_fss = new_storage.getFSSInfo('file', new_file_content)

        old_brains = self.conf.getStorageBrainsByUID(old_fss.getUID())
        new_brains = self.conf.getStorageBrainsByUID(new_fss.getUID())

        st_old = [x for x in old_brains if x['name'] == 'file'][0]
        st_new = [x for x in new_brains if x['name'] == 'file'][0]

        self.assertNotEqual(old_fss.getUID(), new_fss.getUID())
        self.assertNotEqual(len(st_old.keys()), 0, str(st_old))
        self.assertNotEqual(len(st_new.keys()), 0, str(st_new))
        self.assertNotEqual(st_old['fs_path'], st_new['fs_path'])
        self.assertEqual(st_old['size'], st_new['size'])
        self.assertNotEqual(st_old['path'], st_new['path'])

        self.logout()

    def testCopyFolderWithImages(self):
        if not self.use_atct:
            # Only ATFolder implements correctly copy/paste for sub objects
            return

        self.loginAsPortalOwner()

        # Create new folder and fill it with some content
        self.portal.invokeFactory(id='test_source_folder', type_name=FOLDER_TYPE)
        self.sf = getattr(self.portal, 'test_source_folder')

        # adding some images
        self.addImageByFileUpload(self.sf, 's1')
        self.addImageByFileUpload(self.sf, 's2')
        self.addImageByFileUpload(self.sf, 's3')
        self.addImageByFileUpload(self.sf, 's4')
        self.addImageByFileUpload(self.sf, 's5')
        self.addImageByFileUpload(self.sf, 's6')

        self.assertEqual(len(self.sf.objectValues()), 6)

        # creating new subfolder
        self.portal.invokeFactory(id='test_target_folder', type_name=FOLDER_TYPE)

        # make sure objects got copied
        obnum = random.choice([1,2,3,4,5,6])

        o_cobject = getattr(self.sf, 's%s' % obnum)
        o_storage = o_cobject.getField('image').getStorage()
        o_fssinfo = o_storage.getFSSInfo('image', o_cobject)
        o_brains = self.conf.getStorageBrainsByUID(o_fssinfo.getUID())

        # copying complete sf folder to test_target_folder
        cb = self.portal.manage_copyObjects(ids=(self.sf.getId(), ))

        # paste into new folder
        self.tf = getattr(self.portal, 'test_target_folder')
        self.assertEqual(len(self.tf.objectValues()), 0)
        self.tf.manage_pasteObjects(cb_copy_data=cb)
        self.assertEqual(len(self.tf.objectValues()), 1)

        # copied folder should contain six items
        self.cf = getattr(self.tf, 'test_source_folder')
        self.assertEqual(len(self.cf.objectValues()), 6)

        # make sure copied folder got new UID
        self.assertNotEqual(self.cf.getPhysicalPath(), self.sf.getPhysicalPath())
        self.assertEqual(self.cf.getId(), self.sf.getId())

        n_cobject = getattr(self.cf, 's%s' % obnum)
        n_storage = n_cobject.getField('image').getStorage()
        n_fssinfo = n_storage.getFSSInfo('image', n_cobject)
        n_brains = self.conf.getStorageBrainsByUID(n_fssinfo.getUID())

        self.assertNotEqual(o_fssinfo.getUID(), n_fssinfo.getUID())
        self.assertNotEqual(len(o_brains), 0, str(o_brains))
        self.assertNotEqual(len(n_brains), 0, str(n_brains))

        # test if scales were copied
        for name in ('image_mini', 'image_thumb'):
            fs_path = [x['fs_path'] for x in o_brains if x['name'] == name][0]
            assert os.path.exists(fs_path)

        for name in ('image_mini', 'image_thumb'):
            fs_path = [x['fs_path'] for x in n_brains if x['name'] == name][0]
            assert os.path.exists(fs_path)

        # FSS must have created new files on filesystem
        for name in ('image', 'image_mini', 'image_thumb'):
            o_brain = [x for x in o_brains if x['name'] == name][0]
            n_brain = [x for x in n_brains if x['name'] == name][0]
            self.assertNotEqual(o_brain['fs_path'], n_brain['fs_path'])
            self.assertEqual(o_brain['size'], n_brain['size'])
            self.assertNotEqual(o_brain['path'], n_brain['path'])

        # testing deletion
        old_s1_uid = self.sf.s1.UID()
        s1_brains = self.conf.getStorageBrainsByUID(old_s1_uid)
        assert len(s1_brains)>0

        self.sf.manage_delObjects(['s1', ])
        assert 's1' not in self.sf.objectIds()
        s1_brains = self.conf.getStorageBrainsByUID(old_s1_uid)
        assert len(s1_brains)==0


        self.logout()

# #############################################################################
# CUT
# #############################################################################
    def testCutContent(self):
        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)
        old_uid = self.file_content.UID()
        old_storage = getattr(self.test_folder, content_id).getField('file').getStorage()
        old_instance = getattr(self.test_folder, content_id)
        old_fss = old_storage.getFSSInfo('file', old_instance)

        # Copy file in another folder
        cb = self.test_folder.manage_cutObjects(ids=(content_id,))

        new_folder_id = 'new_test_folder'
        self.portal.invokeFactory(FOLDER_TYPE, id=new_folder_id)
        new_folder = getattr(self.portal, new_folder_id)

        # Paste
        new_folder.manage_pasteObjects(cb_copy_data=cb)

        # Test source file
        self.failIf(hasattr(self.test_folder, content_id))

        # Test destination file
        new_file_content = getattr(new_folder, content_id)

        # Get file field
        file_field = new_file_content.getField('file')

        # Get file value
        file_value = file_field.get(new_file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(new_file_content), 'application/msword')

        self.assertEquals(new_file_content.UID(), old_uid)

        # more tests here to test that the external content
        # is also copied
        new_storage = file_field.getStorage()
        new_fss = new_storage.getFSSInfo('file', new_file_content)

        st_old = self.conf.getStorageBrainsByUID(old_fss.getUID())[0]
        st_new = self.conf.getStorageBrainsByUID(new_fss.getUID())[0]

        self.assertEqual(old_fss.getUID(), new_fss.getUID())
        self.assertNotEqual(len(st_old.keys()), 0, str(st_old))
        self.assertNotEqual(len(st_new.keys()), 0, str(st_new))
        self.assertEqual(st_old['fs_path'], st_new['fs_path'])
        self.assertEqual(st_old['size'], st_new['size'])

        self.logout()

    def testCutFolderWithFSSContent(self):
        self.loginAsPortalOwner()

        # Create source folder
        src_folder_id = 'src_folder'
        self.portal.invokeFactory(FOLDER_TYPE, id=src_folder_id)
        src_folder = getattr(self.portal, src_folder_id)

        # Create a file in source folder
        src_content_id = 'src_file'
        src_content = self.addFileByFileUpload(src_folder, src_content_id)

        # Keep a reference to this source content
        old_instance = src_content
        old_uid = old_instance.UID()
        old_storage = old_instance.getField('file').getStorage()
        old_fss = old_storage.getFSSInfo('file', old_instance)

        # Copy source folder to another folder
        cb = self.portal.manage_cutObjects(ids=(src_folder_id,))

        dst_folder_id = 'dst_folder'
        self.portal.invokeFactory(FOLDER_TYPE, id=dst_folder_id)
        dst_folder = getattr(self.portal, dst_folder_id)

        # Paste
        dst_folder.manage_pasteObjects(cb_copy_data=cb)

        # Test that source folder has moved
        self.failIf(hasattr(self.portal, src_folder_id))

        # Get new source folder
        new_folder = getattr(dst_folder, src_folder_id)

        # Get new source content
        new_content = getattr(new_folder, src_content_id)

        # Get file field
        file_field = new_content.getField('file')

        # Get file value and test it
        file_value = file_field.get(new_content)
        self.failUnless(isinstance(file_value, VirtualBinary), file_value.__class__)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(new_content), 'application/msword')

        self.assertEquals(new_content.UID(), old_uid)

        # more tests here to test that the external content
        # is also copied
        new_storage = file_field.getStorage()
        new_fss = new_storage.getFSSInfo('file', new_content)

        st_old = self.conf.getStorageBrainsByUID(old_fss.getUID())[0]
        st_new = self.conf.getStorageBrainsByUID(new_fss.getUID())[0]

        self.assertEqual(old_fss.getUID(), new_fss.getUID())
        self.assertNotEqual(len(st_old.keys()), 0, str(st_old))
        self.assertNotEqual(len(st_new.keys()), 0, str(st_new))
        self.assertEqual(st_old['fs_path'], st_new['fs_path'])
        self.assertEqual(st_old['size'], st_new['size'])

        self.logout()

# #############################################################################
# DELETE
# #############################################################################
    def testDeleteContent(self):
        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # Delete file
        self.test_folder.manage_delObjects(ids=[content_id])

        # Test file
        self.failIf(hasattr(self.test_folder, content_id))
        self.logout()

    def testStorageWhenDeleteContent(self):
        from FSSTestCase import STORAGE_PATH

        self.loginAsPortalOwner()

        # Create file
        content_id = 'test_file'
        self.file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # One item in storage path
        self.failUnlessEqual(len(os.listdir(STORAGE_PATH)), 1)

        # Delete file
        self.test_folder.manage_delObjects(ids=[content_id])

        # Zero items in storage path
        self.failUnlessEqual(len(os.listdir(STORAGE_PATH)), 0)

        self.logout()

# #############################################################################
# FIELD OPERATIONS
# #############################################################################
    def testDeleteField(self):
        self.loginAsPortalOwner()

        # Create content
        content_id = 'test_file'
        file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # Get file field
        file_field = file_content.getField('file')

        # Get file value
        file_value = file_field.get(file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(file_content), 'application/msword')

        # Delete file field value
        file_field.set(file_content, 'DELETE_FILE')

        # Get file value
        file_value = file_field.get(file_content)

        self.assertEquals(file_value, '')

    def testModifyField(self):
        self.loginAsPortalOwner()

        # Create content
        content_id = 'test_file'
        file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # Get file field
        file_field = file_content.getField('file')

        # Get file value
        file_value = file_field.get(file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(file_content), 'application/msword')

        # Delete file field value
        new_value = 'example of content'
        file_field.set(file_content, new_value)

        # Get file value
        file_value = file_field.get(file_content)

        # Test value
        self.assertEquals(str(file_value), new_value)

        # Test content type
        self.assertEquals(file_field.getContentType(file_content), 'text/plain')

# #############################################################################
# UID OPERATIONS
# #############################################################################
    def test_setUID(self):
        self.loginAsPortalOwner()

        # Create content
        content_id = 'test_file'
        file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # Get file field
        file_field = file_content.getField('file')

        # Get file value
        file_value = file_field.get(file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(file_content), 'application/msword')

        # Modify UID
        file_content._setUID('dummyUID')
        self.assertEquals(file_content.UID(), "dummyUID")

        # Get file field
        file_field = file_content.getField('file')

        # Get file value
        file_value = file_field.get(file_content)

        # Test filename
        self.assertEquals(file_value.filename, 'word.doc')

        # Test size
        self.assertEquals(file_value.get_size(), 10240)

        # Test content type
        self.assertEquals(file_field.getContentType(file_content), 'application/msword')



# #############################################################################
# VIRTUAL BINARY OPERATIONS
# #############################################################################
    def testVirtualBinaryAbsoluteUrl(self):

        self.loginAsPortalOwner()

        # Create content
        content_id = 'test_file'
        file_content = self.addFileByFileUpload(self.test_folder, content_id)

        # Get file field
        file_field = file_content.getField('file')

        # Get file value
        file_value = file_field.get(file_content)

        # Test absolute_url
        url = '%(instance_url)s/fss_get/%(name)s' % {
            'instance_url': file_content.absolute_url(),
            'name': 'file',
            }
        self.assertEquals(file_value.absolute_url(), url)


# #############################################################################
# Test File Stream Operator
# #############################################################################

    def _testFileStreamIterator(self,start,end):
        """
        unit test about range_filestream_iterator
        """
        # Create content
        from FSSTestCase import CONTENT_PATH

        # Create range file stream iterator
        from iw.fss.FileSystemStorage import \
            range_filestream_iterator

        iterator = range_filestream_iterator(CONTENT_PATH,start,end,mode='rb')
        data=''
        for i in iterator:
            data +=i
        self.assertEqual(len(data) , end - start  ,
                         '%i != %i len is not correct' % \
                                 (len(data),  end  - start))



    def testRangeFileStreamIterator(self):
        """ test range operation of file stream operator """
        #begin of the file
        self._testFileStreamIterator(0,10)
        #middle
        self._testFileStreamIterator(30,1000)
        #end

        self._testFileStreamIterator(500,1023)
        #

    def testRangeSupport(self):
        """
        functionnal test of range support
        """

        self.loginAsPortalOwner()

        # Create content
        content_id = 'test_file'
        file_content = self.addFileByFileUpload(self.test_folder, content_id)
        # Get file field
        file_field = file_content.getField('file')
        file_content = file_field.get(file_content)
        # do an simple request
        e = {'SERVER_NAME':'foo', 'SERVER_PORT':'80', 'REQUEST_METHOD':'GET'}
        out = StringIO()
        resp = HTTPResponse(stdout=out)
        req = HTTPRequest(sys.stdin, e, resp)
        req.RESPONSE = resp
        data = file_content.index_html(req, resp)
        self.failUnless(len(data) == len(file_content) ,
                        'not good lenght data ')

        # now do an range request with one range
        e =  {'SERVER_NAME':'foo',
              'SERVER_PORT':'80',
              'REQUEST_METHOD':'GET',
              'HTTP_RANGE' : 'bytes=0-10' }
        resp = HTTPResponse(stdout=out)
        req = HTTPRequest(sys.stdin, e, resp)
        req.RESPONSE = resp
        data = file_content.index_html(req, resp)
        read_data = ''
        for d in data:
            read_data +=d

        self.failUnless(len(read_data) == 11 ,
                        'not good lenght data <%s>' % len(read_data))

        # now mulitple range
        e =  {'SERVER_NAME':'foo',
              'SERVER_PORT':'80',
              'REQUEST_METHOD':'GET',
              'HTTP_RANGE' : 'bytes=0-10, 50-80' }
        resp = HTTPResponse(stdout=out)
        req = HTTPRequest(sys.stdin, e, resp)
        req.RESPONSE = resp
        data = file_content.index_html(req, resp)

# #############################################################################
# Test CMFEditions compliance
# #############################################################################

    def testCMFEditions(self):
        self.loginAsPortalOwner()
        portal_repository = self.portal_repository
        data_path = self.getDataPath()
        file1_path = os.path.join(data_path, 'word.doc')
        file1 = open(file1_path, 'rb').read()
        file2_path = os.path.join(data_path, 'excel.xls')
        file2 = open(file2_path, 'rb').read()
        image1_path = os.path.join(data_path, 'image.jpg')
        image1 = open(image1_path, 'rb').read()

        content = self.addATFileByFileUpload(self.folder, 'test_file_and_image')
        portal_repository.applyVersionControl(content, comment='save no 1')

        self.updateContent(content, 'file', file2_path)
        self.updateContent(content, 'image', image1_path)
        portal_repository.save(content, comment='save no 2')

        vdata = portal_repository.retrieve(content, 0)
        obj = vdata.object
        # Verify
        file_field = obj.getField('file')
        file_value = file_field.get(obj)
        self.assertEquals(file_value.filename, 'word.doc')
        self.assertEquals(file_value.get_size(), 10240)
        self.assertEquals(file_field.getContentType(obj), 'application/msword')

        vdata = portal_repository.retrieve(content, 1)
        obj = vdata.object
        # Verify
        file_field = obj.getField('file')
        file_value = file_field.get(obj)
        self.assertEquals(file_value.filename, 'excel.xls')
        self.assertEquals(file_value.get_size(), 13824)
        self.assertEquals(file_field.getContentType(obj), 'application/vnd.ms-excel')

        portal_repository.revert(content, 0)
        # Verify
        file_field = content.getField('file')
        file_value = file_field.get(content)
        self.assertEquals(file_value.filename, 'word.doc')
        self.assertEquals(file_value.get_size(), 10240)
        self.assertEquals(file_field.getContentType(content), 'application/msword')

        self.logout()

# Test all content metadata
strategies = (
    ('FlatStorageStrategy', 'from iw.fss.strategy import FlatStorageStrategy'),
    ('DirectoryStorageStrategy', 'from iw.fss.strategy import DirectoryStorageStrategy'),
    ('SiteStorageStrategy', 'from iw.fss.strategy import SiteStorageStrategy'),
    ('SiteStorageStrategy2', 'from iw.fss.strategy import SiteStorageStrategy2'),
    )


dynamic_class = """
%(import)s

class Test%(name)s(TestFSS):
    "Test fss"

    strategy_klass = %(name)s
"""

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()

    module = sys.modules[TestFSS.__module__]

    # Build dynamic test cases
    for strategy in strategies:
        code = dynamic_class % {
            'name' : strategy[0],
            'import' : strategy[1],
            }

        exec code in module.__dict__
        suite.addTest(makeSuite(getattr(module, 'Test%s' % strategy[0])))

    return suite

if __name__ == '__main__':
    framework()



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""FileSystemStorage extensions 

$Id: __init__.py 43824 2007-06-15 17:08:16Z glenfant $
"""

__author__ = ''
__docformat__ = 'restructuredtext'



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$Id: rdf.py 61129 2008-03-25 16:00:44Z glenfant $
"""

__author__  = ''
__docformat__ = 'restructuredtext'

# Python imports
import re
from xml.sax.saxutils import escape as basic_xml_escape
from xml.sax.handler import ContentHandler
from xml.sax import parseString
from types import StringType, UnicodeType

# Quote strings in rdf
def xml_quote(text):
    """XML quoting as recommended in http://dublincore.org/documents/dcmes-xml/"""

    if text not in (StringType, UnicodeType):
        return text
    return basic_xml_escape(text, {"'": '&apos;', '"': '&quot;'})

# Constants used by RDF writer
RDF_HEADER = """<?xml version="1.0" encoding="%(charset)s"?>"""
RDF_BODY = """<rdf:RDF%(attrs)s>%(items)s</rdf:RDF>"""
RDF_NAMESPACE = '%(id)s="%(value)s"'
RDF_DESCRIPTION = """<rdf:Description rdf:about="%(field_url)s">%(items)s</rdf:Description>"""
RDF_PROP = """<%(id)s>%(value)s</%(id)s>"""

# This class helps you to create easily rdf
class RDFWriter:
    def __init__(self, charset, namespaces, field_url, field_props):
        """Initialize RDF Builder

        @param charset: charset of RDF
        @param namespaces: namespaces used in the used in RDF.
                           List of dict {'id' : ..., 'value', ...}
        @param field_url: field url
        @param field_props: Field properties added in RDF description.
                            List of dict {'id' : ..., 'value', ...}
        """

        self.charset = charset
        self.namespaces = namespaces
        self.field_url = field_url
        self.field_props = field_props

    def getRDF(self):
        """Returns a rdf text rendered from constructor parameters"""

        rdf = ''

        # Add header
        rdf += RDF_HEADER % {'charset': self.charset,}

        # Add namespaces
        namespaces_string = ''
        if self.namespaces:
            namespaces_string = ' %s' % '\n         '.join([RDF_NAMESPACE % {'id': x['id'], 'value': x['value']} for x in self.namespaces])

        # Add rdf items
        field_props_string = ''
        if self.field_props:
            field_props_string = '\n    '.join([RDF_PROP % {'id': x['id'], 'value': xml_quote(x['value'])} for x in self.field_props])
            field_props_string = '\n    %s\n  ' % field_props_string

        # Add rdf description
        rdf_description = RDF_DESCRIPTION % {'field_url': self.field_url, 'items': field_props_string}
        rdf_description = '\n  %s\n' % rdf_description
        rdf_body = RDF_BODY % {'attrs': namespaces_string, 'items': rdf_description}
        rdf_body = '\n%s' % rdf_body
        rdf += rdf_body
        return rdf

# This class helps you to read easily rdf
FIND_ENCODING = r"<\?xml[^>]+encoding=['\"]([-\w]+)['\"]"
FIND_ENCODING_RE = re.compile(FIND_ENCODING)

class RDFHandler(ContentHandler):

    def __init__(self):
        self.info = {}
        self.info['namespaces'] = []
        self.info['field_url'] = ''
        self.info['field_props'] = []
        self.current_node = ''
        self.current_value = ''

    def characters(self, content):
        if self.current_node:
            self.current_value += content

    def startElement(self, name, attrs):
        if name == 'rdf:RDF':
            # Get namespaces
            self.info['namespaces'] = [{'id': k, 'value': v} for k, v in attrs.items()]
        elif name == 'rdf:Description':
            for k, v in attrs.items():
                if k == 'rdf:about':
                    self.info['field_url'] = v
        else:
            self.current_node = name

    def endElement(self, name):
        if name == self.current_node:
            self.info['field_props'].append({'id': self.current_node, 'value': self.current_value})

        self.current_node = ''
        self.current_value = ''

class RDFReader:
    def __init__(self, rdf_text):
        """Initialize RDF Reader

        @param rdf_text: RDF text
        """

        self.rdf_text = rdf_text

        # Parse rdf
        self.rdf_info = self._parseRDF()

    def _parseRDF(self):
        """Parse RDF text"""
        result = {}

        # Get encoding
        charset = None
        match = FIND_ENCODING_RE.match(self.rdf_text)
        if match:
            charset = match.group(1)
        result['charset'] = charset

        # Parse XML
        handler = RDFHandler()
        parseString(self.rdf_text, handler)
        result.update(handler.info)
        return result

    def getRDFInfo(self):
        """Returns all parsed information about RDF"""

        return self.rdf_info

    def getCharset(self):
        """Returns charset of RDF"""

        return self.rdf_info['charset']

    def getNamespaces(self):
        """Returns namespaces used in RDF.

        Returns a list of dict {'id' : ..., 'value', ...}"""

        return self.rdf_info['namespaces']

    def getFieldUrl(self):
        """Returns field url in RDF"""

        return self.rdf_info['field_url']

    def getFieldProperties(self):
        """Returns namespaces used in RDF

        Returns a list of dict {'id' : ..., 'value', ...}"""

        return self.rdf_info['field_props']

    def getFieldProperty(self, property):
        """Returns namespaces used in RDF

        Returns a list of value.

        @param property: Id of the property"""

        props = self.rdf_info['field_props']
        result = []
        for prop in props:
            if prop['id'] == property:
                result.append(prop['value'])
        return result



# -*- coding: utf-8 -*-
## FileSystemStorage
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
The FileSystemStorage tool
$Id: conffile.py 66391 2008-06-09 17:38:35Z glenfant $
"""

__version__ = "$Revision$"
__docformat__ = 'restructuredtext'

# Python imports
import time

# Zope imports
from zope.interface import implements
#getSite return the context-dependent plone site
from zope.app.component.hooks import getSite
# CMF imports
from Products.CMFCore.utils import getToolByName

# Products imports
from iw.fss.utils import rm_file
from iw.fss.FileSystemStorage import FileSystemStorage
from iw.fss.utils import getFieldValue
from iw.fss.config import ZCONFIG
from iw.fss import strategy as fss_strategy
from iw.fss.interfaces import IConf

# {storage-strategy (from config file): strategy class, ...}
_strategy_map = {
    'flat': fss_strategy.FlatStorageStrategy,
    'directory': fss_strategy.DirectoryStorageStrategy,
    'site1': fss_strategy.SiteStorageStrategy,
    'site2': fss_strategy.SiteStorageStrategy2
    }


class ConfFile(object):
    """Tool for FileSystem storage"""

    implements(IConf)


    @property
    def fssPropertySheet(self):

        portal_properties = getToolByName(getSite(), 'portal_properties')
        return portal_properties.filesystemstorage_properties


    def isRDFEnabled(self):
        """Returns true if RDF is automaticaly generated when file added"""

        return bool(self.fssPropertySheet.rdf_enabled)


    def enableRDF(self, enabled):
        """Enable rdf or not"""

        self.fssPropertySheet.rdf_enabled = bool(enabled)

    rdfEnabled = property(isRDFEnabled, enableRDF)


    def getRDFScript(self):
        """Returns rdf script used to generate RDF on files"""

        return self.fssPropertySheet.rdf_script


    def setRDFScript(self, rdf_script):
        """Set rdf script used to generate RDF on files"""

        self.fssPropertySheet.rdf_script = rdf_script.strip()

    rdfScript = property(getRDFScript, setRDFScript)

    def usesGlobalConfig(self):
        """If the global configuration is in use for this site"""

        portal = getSite()
        return ZCONFIG.usesGlobalConfig(portal)

    def getStorageStrategy(self):
        """Returns the storage strategy"""

        global _strategy_map
        portal = getSite()
        portal_path = '/'.join(portal.getPhysicalPath())
        strategy_class = _strategy_map[ZCONFIG.storageStrategyForSite(portal_path)]
        return strategy_class(
            ZCONFIG.storagePathForSite(portal_path),
            ZCONFIG.backupPathForSite(portal_path))


    def getUIDToPathDictionnary(self):
        """Returns a dictionnary

        For one uid (key) give the correct path (value)
        """

        ctool = getToolByName(getSite(), 'uid_catalog')
        brains = ctool(REQUEST={})
        return dict([(x['UID'], x.getPath()) for x in brains])


    def getPathToUIDDictionnary(self):
        """Returns a dictionnary

        For one path (key) give the correct UID (value)
        """

        ctool = getToolByName(getSite(), 'uid_catalog')
        brains = ctool(REQUEST={})
        return dict([(x.getPath(), x['UID']) for x in brains])


    def getFSSBrains(self, items):
        """Returns a dictionnary.

        For one uid, returns a dictionnary containing of fss item stored on
        filesystem:
        - uid: UID of content
        - path: Path of content
        - name: Name of field stored on filesystem
        - size: Size in octets of field value stored on filesystem
        - fs_path: Path on filesystem where the field value is stored
        """

        if not items:
            return []

        # Get the first item of items list and check if item has uid or path key
        if not items[0].has_key('uid'):
            # Use path to uid dictionnary
            path_to_uid = self.getPathToUIDDictionnary()
            for item in items:
                item['uid'] = path_to_uid.get(item['path'], None)
        else:
            # Use uid to path dictionnary
            uid_to_path = self.getUIDToPathDictionnary()
            for item in items:
                item['path'] = uid_to_path.get(item['uid'], None)

        return items


    def getStorageBrains(self):
        """Returns a list of brains in storage path"""

        strategy = self.getStorageStrategy()
        items = strategy.walkOnStorageDirectory()
        return self.getFSSBrains(items)


    def getStorageBrainsByUID(self, uid):
        """ Returns a list containing all brains related to fields stored
        on filesystem of object having the specified uid"""

        return [x for x in self.getStorageBrains() if x['uid'] == uid]


    def getBackupBrains(self):
        """Returns a list of brains in backup path"""

        strategy = self.getStorageStrategy()
        items = strategy.walkOnBackupDirectory()
        return self.getFSSBrains(items)


    def updateFSS(self):
        """
        Update FileSystem storage
        """

        storage_brains = self.getStorageBrains()
        backup_brains = self.getBackupBrains()

        not_valid_files = tuple([x for x in storage_brains if x['path'] is None])
        not_valid_backups = tuple([x for x in backup_brains if x['path'] is not None])
        strategy = self.getStorageStrategy()

        # Move not valid files in backup
        for item in not_valid_files:
            strategy.unsetValueFile(**item)

        # Move not valid backups in file storage
        for item in not_valid_backups:
            strategy.restoreValueFile(**item)
        return len(not_valid_files), len(not_valid_backups)


    def removeBackups(self, max_days):
        """
        Remove backups older than specified days
        """

        backup_brains = self.getBackupBrains()
        valid_backups = [x for x in backup_brains if x['path'] is None]
        current_time = time.time()

        for item in valid_backups:
            one_day = 86400 # One day 86400 seconds
            modified = item['modified']
            seconds = int(current_time) - int(modified.timeTime())
            days = int(seconds/one_day)

            if days >= max_days:
                rm_file(item['fs_path'])


    def updateRDF(self):
        """Add RDF files to fss files"""

        rdf_script = self.getRDFScript()
        storage_brains = self.getStorageBrains()
        strategy = self.getStorageStrategy()

        for item in storage_brains:
            instance_path = item['path']
            if instance_path is None:
                continue

            try:
                instance = self.restrictedTraverse(instance_path)
            except AttributeError:
                # The object doesn't exist anymore, we continue
                continue
            name = item['name']
            field = instance.getField(name)
            if field is None:
                continue
            storage = field.getStorage(instance)
            if not isinstance(storage, FileSystemStorage):
                continue

            # Get FSS info
            info = storage.getFSSInfo(name, instance)
            if info is None:
                continue

            # Call the storage strategy
            rdf_value = info.getRDFValue(name, instance, rdf_script=rdf_script)
            strategy.setRDFFile(rdf_value, uid=item['uid'], name=name)
        return


    def getFSSItem(self, instance, name):
        """Get value of fss item.
        This method is called from fss_get script.

        @param instance: Object containing FSS item
        @param name: Name of FSS item to get
        """

        return getFieldValue(instance, name)



## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id$
"""
GS upgrade steps for iw.fss
"""
__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from zope.app.component.hooks import getSite

def removeFSSTool(setuptool):
    """We don't need it anymore from version 2.7.3"""
    portal = getSite()
    if 'portal_fss' in portal.objectIds():
        portal._delObject('portal_fss')
    return

def addFSSPropertySheet(setuptool):
    """Options are stored in that propertysheet"""

    setuptool.runImportStepFromProfile('profile-iw.fss:default', 'propertiestool',
                                       run_dependencies=False)
    return



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Resources for CMF quick installer
$Id: Install_inactive.py 63643 2008-04-26 16:40:36Z clebeaupin $
"""

__author__  = ''
__docformat__ = 'restructuredtext'

from StringIO import StringIO

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes.public import listTypes

from iw.fss.config import PROJECTNAME, GLOBALS, fss_prefs_configlet
from iw.fss.FSSTool import FSSTool
from iw.fss.modifier import MODIFIER_ID
from iw.fss.modifier import manage_addModifier

def install_modifier(portal, out):
    """Register CMFEditions modifier
    """

    mtool = getToolByName(portal, 'portal_modifier')

    if MODIFIER_ID in  mtool.objectIds():
        out.write("Modifier already installed.")
        return False

    manage_addModifier(mtool)
    out.write("Modifier installed.")
    return True

def install(self):
    out = StringIO()

    # Install types
    type_info = listTypes(PROJECTNAME)
    installTypes(self, out, type_info, PROJECTNAME)

    # Install tools
    add_tool = self.manage_addProduct[PROJECTNAME].manage_addTool
    if not self.objectIds(spec=FSSTool.meta_type):
        add_tool(FSSTool.meta_type)

    # Install skin
    install_subskin(self, out, GLOBALS)

    # Install configlet
    cp_tool = getToolByName(self, 'portal_controlpanel')
    try:
        cp_tool.registerConfiglet(**fss_prefs_configlet)
    except:
        pass

    # Install modifier
    install_modifier(self, out)

    out.write('Installation completed.\n')
    return out.getvalue()

def uninstall(self):
    out = StringIO()

    # Uninstall configlets
    try:
        cp_tool = getToolByName(self, 'portal_controlpanel')
        cp_tool.unregisterApplication(PROJECTNAME)
    except:
        pass

    out.write('Uninstallation completed.\n')
    return out.getvalue()



## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: controlpanel.py 66391 2008-06-09 17:38:35Z glenfant $
"""
Control panel views
"""
__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

#Python imports
import os
import urlparse

# Zope imports
from zope.app.component.hooks import getSite
from zope.component import getUtility
from reStructuredText import HTML
from AccessControl.requestmethod import postonly

# Other components imports
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage

# Product imports
from iw.fss.config import ZCONFIG, CONFIG_FILE
from iw.fss.utils import FSSMessageFactory as _
from iw.fss.interfaces import IConf
from iw.fss.zcml import patchedTypesRegistry


class FSSBaseView(BrowserView):

    def __init__(self, context, request):
        super(FSSBaseView, self).__init__(context, request)
        conf_class = getUtility(IConf, "globalconf")
        self.conf = conf_class()
        return


    def configletTabs(self):
        """Data for drawing tabs in FSS config panel"""

        this_view = urlparse.urlsplit(self.request.URL)[2].split('/')[-1]

        tab_infos = [
            {'label': _(u'management_tab', default=u"Management"),
             'template': 'fss_management_form'},
            {'label': _(u'maintenance_tab', default=u"Maintenance"),
             'template': 'fss_maintenance_form'},
            {'label': _(u'documentation_tab', default=u"Documentation"),
              'template': 'fss_documentation_form'}
            ]

        for ti in tab_infos:
            if ti['template'] == this_view:
                ti['css_class'] = 'selected'
            else:
                ti['css_class'] = None
        return tab_infos

    def rdfEnabled(self):
        return self.conf.rdfEnabled

    def usesGlobalConfig(self):
        """If this site uses the global configuration"""

        return self.conf.usesGlobalConfig()



class FSSManagementView(FSSBaseView):
    """The default management view:
    - plone-filesstemstorage.conf settings
    - TTW options
    """


    def globalConfigInfo(self):
        """A TALES friendly configuration info mapping for global configuration"""

        return {
            'config_file': CONFIG_FILE,
            'strategy': ZCONFIG.storageStrategyForSite('/'),
            'storage_path': ZCONFIG.storagePathForSite('/'),
            'backup_path': ZCONFIG.backupPathForSite('/')
            }


    def siteConfigInfo(self):
        """A TALES friendly configuration info mapping for this Plone site"""

        portal = getSite()
        portal_path = '/'.join(portal.getPhysicalPath())
        return {
            'config_file': CONFIG_FILE,
            'strategy': ZCONFIG.storageStrategyForSite(portal_path),
            'storage_path': ZCONFIG.storagePathForSite(portal_path),
            'backup_path': ZCONFIG.backupPathForSite(portal_path)
            }


    def patchedTypesInfo(self):
        """A TALES friendly summary of content types with storage changed to FSS"""

        out = []
        count = -1
        for type_class, fields_to_storages in patchedTypesRegistry.items():
            count += 1
            feature = {'klass': str(type_class)}
            feature['fields'] = [{'fieldname': fn, 'storage': str(st.__class__)}
                                 for fn, st in fields_to_storages.items()]
            feature['row_css_class'] = ('even', 'odd')[count % 2]
            out.append(feature)
        return out


    def rdfScript(self):
        """Name of ..."""

        return self.conf.rdfScript


    @postonly
    def changeOptions(self, REQUEST):
        """Form handler"""

        rdf_enabled = bool(REQUEST.get('rdf_enabled', None))
        rdf_script = REQUEST.get('rdf_script', '').strip()
        self.conf.rdfEnabled = rdf_enabled
        self.conf.rdfScript = rdf_script
        message = _(
            u'message_properties_saved',
            default=u"Properties saved")
        IStatusMessage(REQUEST).addStatusMessage(message, type='info')
        REQUEST.RESPONSE.redirect(REQUEST.URL1)
        return



class FSSMaintenanceView(FSSBaseView):

    def getFSSStats(self):
        """
        Returns stats on FileSystem storage
        valid_files_count -> Count of valid files
        not_valid_files_count -> Count of not valid files
        valid_backups_count -> Count of valid backups
        not_valid_backups_count -> Count of not valid backups
        """

        storage_brains = self.conf.getStorageBrains()
        backup_brains = self.conf.getBackupBrains()

        valid_files = [x for x in storage_brains if x['path'] is not None]
        not_valid_files = [x for x in storage_brains if x['path'] is None]
        valid_backups = [x for x in backup_brains if x['path'] is None]
        not_valid_backups = [x for x in backup_brains if x['path'] is not None]


        # Sort valid files by size
        def cmp_size(a, b):
              return cmp(a['size'], b['size'])

        valid_files.sort(cmp_size)

        # Size in octets
        total_size = 0
        largest_size = 0
        smallest_size = 0
        average_size = 0

        for x in valid_files:
            total_size += x['size']

        if len(valid_files) > 0:
            largest_size = valid_files[-1]['size']
            smallest_size = valid_files[0]['size']
            average_size = int(total_size / len(valid_files))

        return {
            'valid_files_count' : len(valid_files),
            'not_valid_files_count' : len(not_valid_files),
            'valid_backups_count' : len(valid_backups),
            'not_valid_backups_count' : len(not_valid_backups),
            'total_size' : total_size,
            'largest_size': largest_size,
            'smallest_size' : smallest_size,
            'average_size' : average_size,
            }


    @postonly
    def updateFSS(self, REQUEST):
        """Removed all invalid files"""
        not_valid_files, not_valid_backups = self.conf.updateFSS()
        results = {
            u'not_valid_files': not_valid_files,
            u'not_valid_backups': not_valid_backups}
        message = _(
            u'report_fss_updated',
            default=u"${not_valid_files} files for invalid content moved to backup. ${not_valid_backups} files for invalid backups moved to storage.",
            mapping=results)
        IStatusMessage(REQUEST).addStatusMessage(message, type='info')
        REQUEST.RESPONSE.redirect(REQUEST.URL1)
        return

    @postonly
    def removeBackups(self, REQUEST):
        """Remove backup older than x day"""
        max_days = REQUEST.get("max_days", None)
        if max_days is None:
            max_days = 0
        self.conf.removeBackups(max_days)
        message = _(u'report_old_backups_removed', default=u"Old backups have been removed.")
        IStatusMessage(REQUEST).addStatusMessage(message, type='info')
        REQUEST.RESPONSE.redirect(REQUEST.URL1)
        return

    @postonly
    def updateRDF(self, REQUEST):

        self.conf.updateRDF()
        message = _(u'report_rdf_updated', default=u"RDF files have been updated.")
        IStatusMessage(REQUEST).addStatusMessage(message, type='info')
        REQUEST.RESPONSE.redirect(REQUEST.URL1)
        return



class FSSDocumentationView(FSSBaseView):
    """Shows iw.fss documentation"""

    def readMeHtml(self):
        """Renders README.txt (in rst) inside the page"""

        opd = os.path.dirname
        readme_path = os.path.join(os.path.abspath(opd(opd(__file__))), 'README.txt')
        readme_html = HTML(file(readme_path).read(), report_level=100) # No errors/warnings -> faster
        # Fucking header changes the base href, so we need to tweak all href="#..."
        actual_url = self.request.URL
        return readme_html.replace('href="#', 'href="%s#' % actual_url)



# -*- coding: utf-8 -*-
## FileSystemStorage
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import StringIO
from zope.component import getUtility
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.Field import FileField, StringField, TextField
from Products.Archetypes.Storage import AttributeStorage
from iw.fss.FileSystemStorage import FileSystemStorage
from iw.fss.interfaces import IConf

#
# This method is here for giving an example of storage migration.
# It asserts that you have set storage of FileField of 'Image' and 'File'
# content types to FileSystemStorage. It also asserts that old storage was
# AttributeStorage.
#
# This script is mainly here to give hints about storage migration
# It may not fulfill your needs, but the main idea is here.
# 

def migrateToFSStorage(self):
    """
    Migrate File and Images FileFields to FSStorage
    
    /!\ Assertion is made that the schema definition has been migrated to define
    FSS as storage for interested fields
    """
    try:
        conf = getUtility(IConf, "globalconf")()        
    except AttributeError:
        raise ValueError, "install and configure FileSystemStorage first!"
    
    out = StringIO.StringIO()
    cat = getToolByName(self, 'portal_catalog')
    brains = cat({'portal_type': ['File', 'Image']})

    # this defines are here to avoid instantiating a new one for each file
    # it is known to be harmless with those storages
    attr_storage = AttributeStorage()
    fss_storage = FileSystemStorage()
    
    for b in brains:
        o = b.getObject()
        print >> out, '/'.join(o.getPhysicalPath()), ":",

        # ensure we have an UID
        # it should not happen on a standard plone site, but I've met the case
        # with some weird custom types
        # the UID code was valid on a Plone 2.1.2 bundle (AT 1.3.7)
        if o.UID() is None:
            o._register()
            o._updateCatalog(o.aq_parent)
            print >> out, "UID was None, set to: ", o.UID(),

            
        for f in o.Schema().fields():
            # visit only FileFields with FileSystemStorage
            if not isinstance(f, FileField):
                continue
            storage = f.getStorage()
            if not isinstance(storage, FileSystemStorage):
                continue
            
            name = f.getName()
            print >> out, "'%s'" % name,
            
            # skip if field has already a content
            if f.get_size(o) != 0:
                print >> out, "already set",
                continue

            # get content from old storage and delete old storage
            try:
                content = attr_storage.get(name, o)
            except AttributeError:
                print >> out, "no old value",
                continue
            attr_storage.unset(name, o)
            
            fss_storage.initializeField(o, f) #FIXME: really needed?
            f.set(o, content)

            # unset empty files, this avoid empty files on disk
            if f.get_size(o) == 0:
                print >> out, "unset",
                f.set(o, "DELETE_FILE")
                
            print >> out, ",",

        # new line for next object
        print >> out, ""

    return out.getvalue()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: strategymigrator.py 59863 2008-03-03 13:22:18Z glenfant $

"""%prog [options] SOURCEPATH DESTINATIONPATH

Migrates the storage strategy of a FSS repository in SOURCEPATH
to another strategy in DESTINATIONPATH.

* Purge your FSS storage area using the Plone control panel.
* Activate and re-generate the RDF files if not already done.
  (Before this, reset to standard RDF is a custom RDF is in use)
* Stop your Zope instance.
* Make sure your "umask" and current user/group will let Zope
  process read/write the files we will create in DESTINATIONPATH.
* Run this utility (use"--help" option to read all the doc).
  Note that the DESTINATIONPATH needs free space to hold all
  files of the SOURCEPATH.
* Change your "plone-filesystemstorage.conf" accordingly (RTFM).
* Start your Zope instance and test the content types associated
   with FSS.
* (Backup and) Remove the SOURCEPATH in case of successful tests.
"""

import optparse
import os
import sys
import shutil

__version__ = "1.0.0"
__author__ = "Gilles Lenfant <gilles.lenfant@ingeniweb.com>"

if sys.platform == 'win32':
    import win32file
    def availableSpace(path):
        """Pythonic 'df' in bytes"""

        S, BPS, FC, NC = win32file.GetDiskFreeSpace(path)
        return (long(NC)*long(S)*long(BPS)) / (1024*1024)
else:
    # Unix (Linux, BSD, MacOSX, ...)
    import statvfs
    def availableSpace(path):
        """Pythonic 'df' in bytes"""

        infos = os.statvfs(path)
        free_blocks = infos[statvfs.F_BAVAIL]
        block_size = infos[statvfs.F_FRSIZE]
        return free_blocks * block_size


def usedSpace(path):
    """Pythonic 'du' in bytes"""

    if sys.platform == 'win32':
        # File system supposed to be FAT32 or NTFS
        block_size = 4096
    else:
        # Unix (Linux, BSD, MacOSX, ...)
        infos = os.statvfs(path)
        block_size = infos[statvfs.F_FRSIZE]
    total = 0
    for root, dirs, files in os.walk(path):
        for filename in files:
            filepath = os.path.join(root, filename)
            fsize = os.path.getsize(filepath)
            fsize = ((fsize/block_size) + 1) * block_size # Real size on disk
            total += fsize
    return total


class BaseMigrator(object):
    """Base for all FSS migrators"""

    # These 2 attributes mus be provided by subclasses
    _source_strategy = None
    _destination_strategy = None

    def __init__(self, options, sourcepath, destpath, loginfo):
        """Migrator constructor
        @param options: options as provided by an optparse.OptionParser obj
        @param sourcepath: path to the original FSS storage
        @param destpath: path to the destination FSS storage
        @param loginfo: callable that logs messages
        """

        self.options = options
        self.sourcepath = sourcepath
        self.destpath = destpath
        self.loginfo = loginfo
        return


    def run(self):
        """Execution of migration must be provided by subclasses"""

        raise NotImplementedError


class FlatToDirectoryMigrator(BaseMigrator):
    """storage-strategy flat -> directory"""

    _source_strategy = 'flat'
    _destination_strategy = 'directory'

    def run(self):
        """Go baby, go..."""

        self.loginfo(
            1, "Transforming FSS flat storage in %s into directory storage in %s"
            % (self.sourcepath, self.destpath))
        for filename in os.listdir(self.sourcepath):
            self.loginfo(2, "Processing source file '%s'" % filename)
            dir1 = filename[:2]
            dir2 = filename[:4]
            file_destdir = os.path.join(self.destpath, dir1, dir2)
            if not os.path.isdir(file_destdir):
                self.loginfo(2, "Creating '%s' directory" % file_destdir)
                os.makedirs(file_destdir)
            self.loginfo(2, "Copying '%s' to %s" % (filename, file_destdir))
            file_sourcepath = os.path.join(self.sourcepath, filename)
            shutil.copy(file_sourcepath, file_destdir)
            if self.options.remove_source:
                self.loginfo(2, "Deleting original '%s'" % filename)
                os.remove(file_sourcepath)
        self.loginfo(1, "Done")
        return


class DirectoryToFlatMigrator(BaseMigrator):
    """storage-strategy directory -> flat"""

    _source_strategy = 'directory'
    _destination_strategy = 'flat'

    def run(self):
        """Let's go"""

        self.loginfo(
            1, "Transforming FSS directory storage in %s into flat storage in %s"
            % (self.sourcepath, self.destpath))
        for root, dirnames, filenames in os.walk(self.sourcepath):
            for filename in filenames:
                self.loginfo(2, "Processing source file '%s'" % filename)
                file_sourcepath = os.path.join(root, filename)
                shutil.copy(file_sourcepath, self.destpath)
                if self.options.remove_source:
                    self.loginfo(2, "Deleting original '%s'" % filename)
                    os.remove(file_sourcepath)
        self.loginfo(1, "Done")
        return


_migrator_classes = (FlatToDirectoryMigrator, DirectoryToFlatMigrator)


def allStrategies():
    """Set of all known strategies"""

    out = set()
    for klass in _migrator_classes:
        out |= set([klass._source_strategy, klass._destination_strategy])
    return out


def findMigrator(source_strat, dest_strat):
    """Finds the appropriate migrator class or None"""

    for klass in _migrator_classes:
        if (klass._source_strategy, klass._destination_strategy) == (source_strat, dest_strat):
            return klass
    return None


def availableMigrations():
     """For UI and documentation"""

     out = "\nAvailable startegy migrations:"
     for klass in _migrator_classes:
         out += ("\n* --source-strategy=%s --destination-strategy=%s"
                 % (klass._source_strategy, klass._destination_strategy))
     return out


class Application(object):
    """Shell application"""

    def __init__(self):
        """Getting and validating args and options"""

        parser = optparse.OptionParser(usage=__doc__ + availableMigrations(),
                                       version=__version__)
        parser.add_option(
            '-v', '--verbose', action='count', dest='verbosity', default=0,
            help="Add output verbosity for each -v.")
        parser.add_option(
            '-s', '--source-strategy', action='store', type='string', dest='source_strategy', default='flat', metavar='STRATEGY',
            help=("The FSS strategy used in SOURCEPATH, "
                  "one of '%s'. Default: '%%default'."
                  % "', '".join(allStrategies())))
        parser.add_option(
            '-d', '--destination-strategy', action='store', type='string', dest='destination_strategy', default='directory', metavar='STRATEGY',
            help=("The FSS strategy used in DESTINATIONPATH, "
                  "one of '%s'. Default: '%%default'."
                  % "', '".join(allStrategies())))
        parser.add_option(
            '-r', '--remove-source', action='store_true', dest='remove_source', default=False,
            help=("Remove source files successfully migrated "
                  "(Make a backup of SOURCEPATH before using this option). "
                  "Default: %default."))
        self.options, paths = parser.parse_args()

        # Controlling arguments validity
        if len(paths) != 2:
            parser.print_help()
            print "Error: SOURCEPATH and DESTINATIONPATH are required arguments."
            sys.exit(2)
        self.sourcepath, self.destpath = paths
        if not os.path.isdir(self.sourcepath):
            parser.print_help()
            print "Error: SOURCEPATH is not an existing directory."
            sys.exit(2)
        if not os.path.isdir(self.destpath):
            parser.print_help()
            print "Error: DESTINATIONPATH is not an existing directory."
            sys.exit(2)
        if len(os.listdir(self.destpath)) > 0:
            parser.print_help()
            print "Error: DESTINATIONPATH is not empty."
            sys.exit(2)

        # Finding and validating migrator
        migrator = findMigrator(self.options.source_strategy, self.options.destination_strategy)
        if migrator is None:
            parser.print_help()
            print "Error: This strategy migration is not available."
            sys.exit(2)
        self.migrator = migrator

        # Checking available space
        if not self.options.remove_source:
            used = usedSpace(self.sourcepath)
            if availableSpace(self.destpath) < used:
                print ("Error: there is not enough free space in '%s'. "
                       "You need %d free bytes in the destination path, "
                       "or consider making a backup and use the "
                       "'--remove-source' option." % (self.destpath, used))
                sys.exit(2)
            else:
                self.loginfo(
                    1,
                    ("Migration will take approximately %d bytes "
                     "in %s" % (used, self.destpath)))
        return


    def run(self):
        """Let's do the job"""

        migrator = self.migrator(self.options, self.sourcepath, self.destpath, self.loginfo)
        migrator.run()
        return


    def loginfo(self, level, message):
        """Prints the message from the verbosity level"""

        if self.options.verbosity >= level:
            print message
        return


if __name__ == '__main__':
    Application().run()



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: base.py 66391 2008-06-09 17:38:35Z glenfant $

"""Defines a test class and its Plone Site layer for plone tests"""

import os
import Globals

from Testing import ZopeTestCase as ztc

from zope.interface import classImplements
from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.CMFPlone.Portal import PloneSite
from Products.CMFPlone.interfaces import ITestCasePloneSiteRoot
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase.layer import onsetup

import iw.fss

iw.fss.config.ZOPETESTCASE = True
from iw.fss.config import INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE

# Install FSS Example types
os.environ[INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE] = 'True'

# Make the test fixture extension profile active
classImplements(PloneSite, ITestCasePloneSiteRoot)

STORAGE_PATH = os.path.join(Globals.INSTANCE_HOME, 'var', 'unittests_storage')
BACKUP_PATH = os.path.join(Globals.INSTANCE_HOME, 'var', 'unittests_backup')

@onsetup
def setup_fss():
    """Set up the additional products required for fss.

    The @onsetup decorator causes the execution of this body to be deferred
    until the setup of the Plone site testing layer.
    """
    ztc.installPackage('iw.fss')

def createTemporaryDirs():
    # Create temporary dirs to run test cases
    for base_path in (STORAGE_PATH, BACKUP_PATH):
        if not os.path.exists(base_path):
            os.mkdir(base_path)

def removeTemporaryDirs():
    # removes previoulys created dirs
    import shutil
    shutil.rmtree(STORAGE_PATH)
    shutil.rmtree(BACKUP_PATH)
    return

# setting up plone site
setup_fss()
ptc.setupPloneSite(products=['iw.fss'],
                   extension_profiles=['iw.fss:default', 'iw.fss:testfixtures'])

# fake mailhost
from Products.MailHost import MailHost

class TestMailHost(MailHost.MailHost):

    def _send(self, mfrom, mto, messageText ):
        """Fake sender"""
        print messageText

class TestCase(ptc.FunctionalTestCase):
    """test case used in tests"""

    class layer(PloneSite):
        @classmethod
        def setUp(cls):
            fiveconfigure.debug_mode = True
            zcml.load_config('configure.zcml', iw.fss)
            fiveconfigure.debug_mode = False
            cls._old = MailHost.MailHost
            MailHost.MailHost = TestMailHost
            createTemporaryDirs()

        @classmethod
        def tearDown(cls):
            MailHost.MailHost = cls._old
            removeTemporaryDirs()



# -*- coding: utf-8 -*-
## Copyright (C) 2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Testing configuration schema and default configuration file
$Id: test_configuration.py 66025 2008-06-02 13:53:50Z glenfant $
"""

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import unittest
import os
from StringIO import StringIO
import ZConfig

import iw.fss.customconfig
iw.fss.customconfig.ZOPETESTCASE = True

from iw.fss.configuration import datatypes

# Test configurations
GOOD_CONFIG1 = """# Two distinct directories
storage-path $$INSTANCE_HOME/var
backup-path $$INSTANCE_HOME/etc
storage-strategy directory
"""

GOOD_CONFIG2 = """# Two distinct directories, two plone sites
storage-path $$INSTANCE_HOME/lib
backup-path $$INSTANCE_HOME/var
# default storage-strategy (flat)
<site /foo/bar>
  storage-path $$INSTANCE_HOME/bin
  backup-path $$INSTANCE_HOME/log
  storage-strategy directory
</site>
<site /YO/stuff>
  storage-path $$INSTANCE_HOME/etc
  backup-path $$INSTANCE_HOME/Products
  # default storage-strategy (flat)
</site>
"""

GOOD_CONFIG3 = """# Empty config file ;-)
# Assumes that $INSTANCE/var/fss_storage and $INSTANCE/var/fss_backup
"""

_good_configs = (GOOD_CONFIG1, GOOD_CONFIG2, GOOD_CONFIG3)

BAD_CONFIG1 = """# Non existing storage directory
storage-path /foo/bar
"""

BAD_CONFIG2 = """# No write access to storage directory (Unix)
storage-path /etc
"""

BAD_CONFIG3 = """# Duplicated directories
storage-path $$INSTANCE_HOME/var/fss_storage
backup-path $$INSTANCE_HOME/var/fss_storage
"""

BAD_CONFIG4 = """# No such strategy
storage-strategy foo
"""

BAD_CONFIG5 = """# site1 default strategy forbidden
storage-strategy site1"""

_bad_configs = (BAD_CONFIG1, BAD_CONFIG2, BAD_CONFIG3, BAD_CONFIG4, BAD_CONFIG5)


class ConfigSchemaTest(unittest.TestCase):
    """Testing configuration schema conformance"""


    def testSchemaConformance(self):
        """Our schema.xml conforms ZConfig schema"""

        from iw.fss.configuration import schema
        self.schema = schema.fssSchema
        # self.schema = ZConfig.loadSchema(self.schema_path)
        return


class BaseConfigTest(unittest.TestCase):
    """Common resources for testing FSS configuration"""

    def setUp(self):

        from iw.fss.configuration import schema
        self.schema = schema.fssSchema
        return


class ConfigFilesValidityTest(BaseConfigTest):
    """Testing some configuration files"""


    def testGoodConfigLoad(self):
        """A bunch of correct configuration files"""

        global _good_configs
        for config in _good_configs:
            good_config = StringIO(config)
            conf, handler = ZConfig.loadConfigFile(self.schema, good_config)
            datatypes._paths = []
        return

    def testBadConfigLoad(self):
        """A bunch of incorrect configuration files"""

        global _bad_configs
        for config in _bad_configs:
            bad_config = StringIO(config)
            self.failUnlessRaises(Exception, ZConfig.loadConfigFile, self.schema, bad_config)
            datatypes._paths = []
        return

class ConfigObjectTest(BaseConfigTest):
    """Testing configuration classes"""

    def setUp(self):

        super(ConfigObjectTest, self).setUp()
        self.zconf, handler = ZConfig.loadConfigFile(self.schema, StringIO(GOOD_CONFIG2))
        datatypes._paths = []
        return

    def testGlobalAttrs(self):
        """Attributes of global config"""

        self.assertEqual(self.zconf.storage_path, os.path.expandvars('$INSTANCE_HOME/lib'))
        self.assertEqual(self.zconf.backup_path, os.path.expandvars('$INSTANCE_HOME/var'))
        self.assertEqual(self.zconf.storage_strategy, 'flat')
        self.assertEqual(len(self.zconf.sites), 2)
        return

    def testSitesAttrs(self):
        """Attributes of site specific settings"""

        expected = {'storage_path': os.path.expandvars('$INSTANCE_HOME/bin'),
                    'backup_path': os.path.expandvars('$INSTANCE_HOME/log'),
                    'storage_strategy': 'directory',
                    'name': '/foo/bar'}
        self._testSiteAttrs(self.zconf.sites[0], expected)

        expected = {'storage_path': os.path.expandvars('$INSTANCE_HOME/etc'),
                    'backup_path': os.path.expandvars('$INSTANCE_HOME/Products'),
                    'storage_strategy': 'flat',
                    'name': '/YO/stuff'}
        self._testSiteAttrs(self.zconf.sites[1], expected)
        return

    def _testSiteAttrs(self, siteconf, expected):
        for attrname, value in expected.items():
            if attrname == 'name':
                self.assertEqual(getattr(siteconf, attrname), value.lower())
            else:
                self.assertEqual(getattr(siteconf, attrname), value)
        return

    def testConfigServices(self):
        """Canonical config API"""

        # Default config for /any/site
        self.assertEqual(self.zconf.storagePathForSite('/any/site'),
                         os.path.expandvars('$INSTANCE_HOME/lib'))
        self.assertEqual(self.zconf.backupPathForSite('/any/site'),
                         os.path.expandvars('$INSTANCE_HOME/var'))
        self.assertEqual(self.zconf.storageStrategyForSite('/any/site'), 'flat')

        # /foo/bar specific config
        self.assertEqual(self.zconf.storagePathForSite('/foo/bar'),
                         os.path.expandvars('$INSTANCE_HOME/bin'))
        self.assertEqual(self.zconf.backupPathForSite('/foo/bar'),
                         os.path.expandvars('$INSTANCE_HOME/log'))
        self.assertEqual(self.zconf.storageStrategyForSite('/foo/bar'), 'directory')

        # /YO/stuff specific config
        self.assertEqual(self.zconf.storagePathForSite('/YO/stuff'),
                         os.path.expandvars('$INSTANCE_HOME/etc'))
        self.assertEqual(self.zconf.backupPathForSite('/YO/stuff'),
                         os.path.expandvars('$INSTANCE_HOME/Products'))
        self.assertEqual(self.zconf.storageStrategyForSite('/YO/stuff'), 'flat')
        return


class DefaultConfigTest(unittest.TestCase):
    """Checking the default configuration file
    (etc/plone-filesystemstorage.conf.in)"""

    def testDefaultConfig(self):
        from iw.fss.config import ZCONFIG

        self.assertEqual(ZCONFIG.storagePathForSite('/any/site'),
                         os.path.normpath(os.path.expandvars('$INSTANCE_HOME/var/fss_storage')))
        self.assertEqual(ZCONFIG.backupPathForSite('/any/site'),
                         os.path.normpath(os.path.expandvars('$INSTANCE_HOME/var/fss_backup')))
        self.assertEqual(ZCONFIG.storageStrategyForSite('/any/site'), 'flat')
        self.assertEqual(len(ZCONFIG.sites), 0)
        return


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ConfigSchemaTest))
    suite.addTest(makeSuite(ConfigFilesValidityTest))
    suite.addTest(makeSuite(ConfigObjectTest))
    suite.addTest(makeSuite(DefaultConfigTest))
    return suite
